@@ -10,7 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""Time-based One-time Password Algorithm (TOTP) auth plugin
+"""Time-based One-time Password Algorithm (TOTP) auth plugin.
 
 TOTP is an algorithm that computes a one-time password from a shared secret
 key and the current time.
@@ -31,9 +31,9 @@ from oslo_log import log
 from oslo_utils import timeutils
 import six
 
-from keystone import auth
 from keystone.auth import plugins
-from keystone.common import dependency
+from keystone.auth.plugins import base
+from keystone.common import provider_api
 from keystone import exception
 from keystone.i18n import _
 
@@ -41,6 +41,7 @@ from keystone.i18n import _
 METHOD_NAME = 'totp'
 
 LOG = log.getLogger(__name__)
+PROVIDERS = provider_api.ProviderAPIs
 
 
 def _generate_totp_passcode(secret):
@@ -60,20 +61,24 @@ def _generate_totp_passcode(secret):
         secret = secret + b'='
 
     decoded = base64.b32decode(secret)
+    # NOTE(lhinds) This is marked as #nosec since bandit will see SHA1
+    # which is marked as insecure. In this instance however, keystone uses
+    # HMAC-SHA1 when generating the TOTP, which is currently not insecure but
+    # will still trigger when scanned by bandit.
     totp = crypto_totp.TOTP(
-        decoded, 6, hashes.SHA1(), 30, backend=default_backend())
-    return totp.generate(timeutils.utcnow_ts(microsecond=True))
+        decoded, 6, hashes.SHA1(), 30, backend=default_backend())  # nosec
+    return totp.generate(timeutils.utcnow_ts(microsecond=True)).decode('utf-8')
 
 
-@dependency.requires('credential_api')
-class TOTP(auth.AuthMethodHandler):
+class TOTP(base.AuthMethodHandler):
 
-    def authenticate(self, context, auth_payload, auth_context):
-        """Try to authenticate using TOTP"""
+    def authenticate(self, request, auth_payload):
+        """Try to authenticate using TOTP."""
+        response_data = {}
         user_info = plugins.TOTPUserInfo.create(auth_payload, METHOD_NAME)
         auth_passcode = auth_payload.get('user').get('passcode')
 
-        credentials = self.credential_api.list_credentials_for_user(
+        credentials = PROVIDERS.credential_api.list_credentials_for_user(
             user_info.user_id, type='totp')
 
         valid_passcode = False
@@ -96,4 +101,7 @@ class TOTP(auth.AuthMethodHandler):
             msg = _('Invalid username or TOTP passcode')
             raise exception.Unauthorized(msg)
 
-        auth_context['user_id'] = user_info.user_id
+        response_data['user_id'] = user_info.user_id
+
+        return base.AuthHandlerResponse(status=True, response_body=None,
+                                        response_data=response_data)
