@@ -10,6 +10,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from keystone.common.validation import parameter_types
+
+
 token_issue = {
     'type': 'object',
     'properties': {
@@ -82,6 +85,18 @@ token_issue = {
                         'name': {'type': 'string', },
                     },
                 },
+                'OS-TRUST:trust': {
+                    'type': 'object',
+                    'properties': {
+                        'id': {'type': 'string', },
+                    }
+                },
+                'system': {
+                    'type': 'object',
+                    'properties': {
+                        'all': parameter_types.boolean
+                    }
+                }
             },
         },
     },
