@@ -20,28 +20,32 @@ import calendar
 import collections
 import grp
 import hashlib
+import itertools
 import os
 import pwd
 import uuid
 
-from oslo_config import cfg
 from oslo_log import log
 from oslo_serialization import jsonutils
 from oslo_utils import reflection
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import passlib.hash
 import six
 from six import moves
 
-from keystone.common import authorization
+from keystone.common import password_hashing
+import keystone.conf
 from keystone import exception
-from keystone.i18n import _, _LE, _LW
+from keystone.i18n import _
 
 
-CONF = cfg.CONF
-
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
+WHITELISTED_PROPERTIES = [
+    'tenant_id', 'project_id', 'user_id',
+    'public_bind_host', 'admin_bind_host',
+    'compute_host', 'admin_port', 'public_port',
+    'public_endpoint', 'admin_endpoint', ]
 
 
 # NOTE(stevermar): This UUID must stay the same, forever, across
@@ -49,9 +53,15 @@ LOG = log.getLogger(__name__)
 # used for ID transformation.
 RESOURCE_ID_NAMESPACE = uuid.UUID('4332ecab-770b-4288-a680-b9aca3b1b153')
 
+# Compatibilty for password hashing functions.
+verify_length_and_trunc_password = password_hashing.verify_length_and_trunc_password  # noqa
+hash_password = password_hashing.hash_password
+hash_user_password = password_hashing.hash_user_password
+check_password = password_hashing.check_password
+
 
 def resource_uuid(value):
-    """Converts input to valid UUID hex digits."""
+    """Convert input to valid UUID hex digits."""
     try:
         uuid.UUID(value)
         return value
@@ -65,7 +75,7 @@ def resource_uuid(value):
 
 
 def flatten_dict(d, parent_key=''):
-    """Flatten a nested dictionary
+    """Flatten a nested dictionary.
 
     Converts a dictionary with nested values to a single level flat
     dictionary, with dotted notation for each key.
@@ -81,26 +91,6 @@ def flatten_dict(d, parent_key=''):
     return dict(items)
 
 
-def read_cached_file(filename, cache_info, reload_func=None):
-    """Read from a file if it has been modified.
-
-    :param cache_info: dictionary to hold opaque cache.
-    :param reload_func: optional function to be called with data when
-                        file is reloaded due to a modification.
-
-    :returns: data from file.
-
-    """
-    mtime = os.path.getmtime(filename)
-    if not cache_info or mtime != cache_info.get('mtime'):
-        with open(filename) as fap:
-            cache_info['data'] = fap.read()
-        cache_info['mtime'] = mtime
-        if reload_func:
-            reload_func(cache_info['data'])
-    return cache_info['data']
-
-
 class SmarterEncoder(jsonutils.json.JSONEncoder):
     """Help for JSON encoding dict-like objects."""
 
@@ -110,31 +100,6 @@ class SmarterEncoder(jsonutils.json.JSONEncoder):
         return super(SmarterEncoder, self).default(obj)
 
 
-class PKIEncoder(SmarterEncoder):
-    """Special encoder to make token JSON a bit shorter."""
-
-    item_separator = ','
-    key_separator = ':'
-
-
-def verify_length_and_trunc_password(password):
-    """Verify and truncate the provided password to the max_password_length."""
-    max_length = CONF.identity.max_password_length
-    try:
-        if len(password) > max_length:
-            if CONF.strict_password_check:
-                raise exception.PasswordVerificationError(size=max_length)
-            else:
-                LOG.warning(
-                    _LW('Truncating user password to '
-                        '%d characters.'), max_length)
-                return password[:max_length]
-        else:
-            return password
-    except TypeError:
-        raise exception.ValidationError(attribute='string', target='password')
-
-
 def hash_access_key(access):
     hash_ = hashlib.sha256()
     if not isinstance(access, six.binary_type):
@@ -143,37 +108,8 @@ def hash_access_key(access):
     return hash_.hexdigest()
 
 
-def hash_user_password(user):
-    """Hash a user dict's password without modifying the passed-in dict."""
-    password = user.get('password')
-    if password is None:
-        return user
-
-    return dict(user, password=hash_password(password))
-
-
-def hash_password(password):
-    """Hash a password. Hard."""
-    password_utf8 = verify_length_and_trunc_password(password).encode('utf-8')
-    return passlib.hash.sha512_crypt.encrypt(
-        password_utf8, rounds=CONF.crypt_strength)
-
-
-def check_password(password, hashed):
-    """Check that a plaintext password matches hashed.
-
-    hashpw returns the salt value concatenated with the actual hash value.
-    It extracts the actual salt if this value is then passed as the salt.
-
-    """
-    if password is None or hashed is None:
-        return False
-    password_utf8 = verify_length_and_trunc_password(password).encode('utf-8')
-    return passlib.hash.sha512_crypt.verify(password_utf8, hashed)
-
-
 def attr_as_boolean(val_attr):
-    """Returns the boolean value, decoded from a string.
+    """Return the boolean value, decoded from a string.
 
     We test explicitly for a value meaning False, which can be one of
     several formats as specified in oslo strutils.FALSE_STRINGS.
@@ -184,6 +120,23 @@ def attr_as_boolean(val_attr):
     return strutils.bool_from_string(val_attr, default=True)
 
 
+def remove_duplicate_dicts_by_id(item_list):
+    """Return a list with duplicate items removed.
+
+    This compares duplicates based on item id. This is to account
+    for dictionaries of items that can contain values, such as a
+    list of tags, that will not be guaranteed to be in order.
+
+    :param item_list: a list of dictionaries
+    :returns: a list of unique dictionaries
+
+    """
+    unique = {}
+    for i in item_list:
+        unique[i['id']] = i
+    return list(unique.values())
+
+
 def get_blob_from_credential(credential):
     try:
         blob = jsonutils.loads(credential.blob)
@@ -217,7 +170,7 @@ def convert_v3_to_ec2_credential(credential):
 
 
 def unixtime(dt_obj):
-    """Format datetime object as unix timestamp
+    """Format datetime object as unix timestamp.
 
     :param dt_obj: datetime.datetime object
     :returns: float
@@ -264,10 +217,10 @@ def setup_remote_pydev_debug():
                             stderrToServer=True)
             return True
         except Exception:
-            LOG.exception(_LE(
+            LOG.exception(
                 'Error setting up the debug environment. Verify that the '
                 'option --debug-url has the format <host>:<port> and that a '
-                'debugger processes is listening on that port.'))
+                'debugger processes is listening on that port.')
             raise
 
 
@@ -498,6 +451,7 @@ class WhiteListedItemFilter(object):
         self._data = data
 
     def __getitem__(self, name):
+        """Evaluation on an item access."""
         if name not in self._whitelist:
             raise KeyError
         return self._data[name]
@@ -508,49 +462,57 @@ _ISO8601_TIME_FORMAT = '%Y-%m-%dT%H:%M:%S'
 
 
 def isotime(at=None, subsecond=False):
-    """Stringify time in ISO 8601 format."""
-    # Python provides a similar instance method for datetime.datetime objects
-    # called isoformat(). The format of the strings generated by isoformat()
-    # have a couple of problems:
-    # 1) The strings generated by isotime are used in tokens and other public
-    #    APIs that we can't change without a deprecation period. The strings
-    #    generated by isoformat are not the same format, so we can't just
-    #    change to it.
-    # 2) The strings generated by isoformat do not include the microseconds if
-    #    the value happens to be 0. This will likely show up as random failures
-    #    as parsers may be written to always expect microseconds, and it will
-    #    parse correctly most of the time.
+    """Stringify time in ISO 8601 format.
 
+    Python provides a similar instance method for datetime.datetime objects
+    called `isoformat()`. The format of the strings generated by `isoformat()`
+    has a couple of problems:
+
+    1) The strings generated by `isotime()` are used in tokens and other public
+    APIs that we can't change without a deprecation period. The strings
+    generated by `isoformat()` are not the same format, so we can't just
+    change to it.
+
+    2) The strings generated by `isoformat()` do not include the microseconds
+    if the value happens to be 0. This will likely show up as random
+    failures as parsers may be written to always expect microseconds, and it
+    will parse correctly most of the time.
+
+    :param at: Optional datetime object to return at a string. If not provided,
+               the time when the function was called will be used.
+    :type at: datetime.datetime
+    :param subsecond: If true, the returned string will represent microsecond
+                      precision, but only precise to the second. For example, a
+                      `datetime.datetime(2016, 9, 14, 14, 1, 13, 970223)` will
+                      be returned as `2016-09-14T14:01:13.000000Z`.
+    :type subsecond: bool
+    :returns: A time string represented in ISO 8601 format.
+    :rtype: str
+    """
     if not at:
-        at = timeutils.utcnow().replace(microsecond=0)
+        at = timeutils.utcnow()
+    # NOTE(lbragstad): Datetime objects are immutable, so reassign the date we
+    # are working with to itself as we drop microsecond precision.
+    at = at.replace(microsecond=0)
     st = at.strftime(_ISO8601_TIME_FORMAT
                      if not subsecond
                      else _ISO8601_TIME_FORMAT_SUBSECOND)
     tz = at.tzinfo.tzname(None) if at.tzinfo else 'UTC'
-    st += ('Z' if tz == 'UTC' else tz)
+    # Need to handle either iso8601 or python UTC format
+    st += ('Z' if tz in ['UTC', 'UTC+00:00'] else tz)
     return st
 
 
-def strtime():
-    at = timeutils.utcnow()
-    return at.strftime(timeutils.PERFECT_TIME_FORMAT)
-
-
-def get_token_ref(context):
-    """Retrieves KeystoneToken object from the auth context and returns it.
-
-    :param dict context: The request context.
-    :raises keystone.exception.Unauthorized: If auth context cannot be found.
-    :returns: The KeystoneToken object.
-    """
+def parse_expiration_date(expiration_date):
+    if not expiration_date.endswith('Z'):
+        expiration_date += 'Z'
     try:
-        # Retrieve the auth context that was prepared by AuthContextMiddleware.
-        auth_context = (context['environment']
-                        [authorization.AUTH_CONTEXT_ENV])
-        return auth_context['token']
-    except KeyError:
-        LOG.warning(_LW("Couldn't find the auth context."))
-        raise exception.Unauthorized()
+        expiration_time = timeutils.parse_isotime(expiration_date)
+    except ValueError:
+        raise exception.ValidationTimeStampError()
+    if timeutils.is_older_than(expiration_time, 0):
+        raise exception.ValidationExpirationError()
+    return expiration_time
 
 
 URL_RESERVED_CHARS = ":/?#[]@!$&'()*+,;="
@@ -571,7 +533,7 @@ def list_url_unsafe_chars(name):
 
 
 def lower_case_hostname(url):
-    """Change the URL's hostname to lowercase"""
+    """Change the URL's hostname to lowercase."""
     # NOTE(gyee): according to
     # https://www.w3.org/TR/WD-html40-970708/htmlweb.html, the netloc portion
     # of the URL is case-insensitive
@@ -585,9 +547,9 @@ def remove_standard_port(url):
     # remove the default ports specified in RFC2616 and 2818
     o = moves.urllib.parse.urlparse(url)
     separator = ':'
-    (host, separator, port) = o.netloc.partition(':')
+    (host, separator, port) = o.netloc.partition(separator)
     if o.scheme.lower() == 'http' and port == '80':
-        # NOTE(gyee): _replace() is not a private method. It has an
+        # NOTE(gyee): _replace() is not a private method. It has
         # an underscore prefix to prevent conflict with field names.
         # See https://docs.python.org/2/library/collections.html#
         # collections.namedtuple
@@ -596,3 +558,65 @@ def remove_standard_port(url):
         o = o._replace(netloc=host)
 
     return moves.urllib.parse.urlunparse(o)
+
+
+def format_url(url, substitutions, silent_keyerror_failures=None):
+    """Format a user-defined URL with the given substitutions.
+
+    :param string url: the URL to be formatted
+    :param dict substitutions: the dictionary used for substitution
+    :param list silent_keyerror_failures: keys for which we should be silent
+        if there is a KeyError exception on substitution attempt
+    :returns: a formatted URL
+
+    """
+    substitutions = WhiteListedItemFilter(
+        WHITELISTED_PROPERTIES,
+        substitutions)
+    allow_keyerror = silent_keyerror_failures or []
+    try:
+        result = url.replace('$(', '%(') % substitutions
+    except AttributeError:
+        msg = "Malformed endpoint - %(url)r is not a string"
+        LOG.error(msg, {"url": url})
+        raise exception.MalformedEndpoint(endpoint=url)
+    except KeyError as e:
+        if not e.args or e.args[0] not in allow_keyerror:
+            msg = "Malformed endpoint %(url)s - unknown key %(keyerror)s"
+            LOG.error(msg, {"url": url, "keyerror": e})
+            raise exception.MalformedEndpoint(endpoint=url)
+        else:
+            result = None
+    except TypeError as e:
+        msg = ("Malformed endpoint '%(url)s'. The following type error "
+               "occurred during string substitution: %(typeerror)s")
+        LOG.error(msg, {"url": url, "typeerror": e})
+        raise exception.MalformedEndpoint(endpoint=url)
+    except ValueError:
+        msg = ("Malformed endpoint %s - incomplete format "
+               "(are you missing a type notifier ?)")
+        LOG.error(msg, url)
+        raise exception.MalformedEndpoint(endpoint=url)
+    return result
+
+
+def check_endpoint_url(url):
+    """Check substitution of url.
+
+    The invalid urls are as follows:
+    urls with substitutions that is not in the whitelist
+
+    Check the substitutions in the URL to make sure they are valid
+    and on the whitelist.
+
+    :param str url: the URL to validate
+    :rtype: None
+    :raises keystone.exception.URLValidationError: if the URL is invalid
+    """
+    # check whether the property in the path is exactly the same
+    # with that in the whitelist below
+    substitutions = dict(zip(WHITELISTED_PROPERTIES, itertools.repeat('')))
+    try:
+        url.replace('$(', '%(') % substitutions
+    except (KeyError, TypeError, ValueError):
+        raise exception.URLValidationError(url)
