@@ -11,12 +11,43 @@
 # under the License.
 """Internal implementation of request body validating middleware."""
 
+import re
+
 import jsonschema
+from oslo_config import cfg
+from oslo_log import log
+import six
 
 from keystone import exception
 from keystone.i18n import _
 
 
+CONF = cfg.CONF
+LOG = log.getLogger(__name__)
+
+
+# TODO(rderose): extend schema validation and add this check there
+def validate_password(password):
+    pattern = CONF.security_compliance.password_regex
+    if pattern:
+        if not isinstance(password, six.string_types):
+            detail = _("Password must be a string type")
+            raise exception.PasswordValidationError(detail=detail)
+        try:
+            if not re.match(pattern, password):
+                pattern_desc = (
+                    CONF.security_compliance.password_regex_description)
+                raise exception.PasswordRequirementsValidationError(
+                    detail=pattern_desc)
+        except re.error:
+            msg = ("Unable to validate password due to invalid regular "
+                   "expression - password_regex: ")
+            LOG.error(msg, pattern)
+            detail = _("Unable to validate password due to invalid "
+                       "configuration")
+            raise exception.PasswordValidationError(detail=detail)
+
+
 class SchemaValidator(object):
     """Resource reference validator class."""
 
@@ -50,9 +81,10 @@ class SchemaValidator(object):
                 # too long, then we should build the masking in here so that
                 # we don't expose sensitive user information in the event it
                 # fails validation.
-                detail = _("Invalid input for field '%(path)s'. The value is "
-                           "'%(value)s'.") % {'path': ex.path.pop(),
-                                              'value': ex.instance}
+                path = '/'.join(map(six.text_type, ex.path))
+                detail = _("Invalid input for field '%(path)s': "
+                           "%(message)s") % {'path': path,
+                                             'message': six.text_type(ex)}
             else:
-                detail = ex.message
+                detail = six.text_type(ex)
             raise exception.SchemaValidationError(detail=detail)
