@@ -35,7 +35,6 @@ import webob.dec
 import webob.exc
 
 from keystone.common import authorization
-from keystone.common import dependency
 from keystone.common import json_home
 from keystone.common import request as request_mod
 from keystone.common import utils
@@ -199,7 +198,6 @@ class BaseApplication(object):
         raise NotImplementedError('You must implement __call__')
 
 
-@dependency.requires('assignment_api', 'policy_api', 'token_provider_api')
 class Application(BaseApplication):
 
     @webob.dec.wsgify(RequestClass=request_mod.Request)
@@ -235,7 +233,10 @@ class Application(BaseApplication):
                                     context=req.context_dict,
                                     user_locale=best_match_language(req))
         except exception.Error as e:
-            LOG.warning(six.text_type(e))
+            if isinstance(e, exception.UnexpectedError):
+                LOG.exception(six.text_type(e))
+            else:
+                LOG.warning(six.text_type(e))
             return render_exception(e,
                                     context=req.context_dict,
                                     user_locale=best_match_language(req))
