@@ -20,7 +20,11 @@ methods = cfg.ListOpt(
     'methods',
     default=constants._DEFAULT_AUTH_METHODS,
     help=utils.fmt("""
-Allowed authentication methods.
+Allowed authentication methods. Note: You should disable the `external` auth
+method if you are currently using federation. External auth and federation
+both use the REMOTE_USER variable. Since both the mapped and external plugin
+are being invoked to validate attributes in the request environment, it can
+cause conflicts.
 """))
 
 password = cfg.StrOpt(  # nosec : This is the name of the plugin, not
@@ -64,6 +68,24 @@ namespace. You do not need to set this unless you are overriding keystone's own
 `oauth1` authentication plugin.
 """))
 
+mapped = cfg.StrOpt(
+    'mapped',
+    help=utils.fmt("""
+Entry point for the mapped auth plugin module in the `keystone.auth.mapped`
+namespace. You do not need to set this unless you are overriding keystone's own
+`mapped` authentication plugin.
+"""))
+
+application_credential = cfg.StrOpt(
+    'application_credential',
+    help=utils.fmt("""
+Entry point for the application_credential auth plugin module in the
+`keystone.auth.application_credential` namespace. You do not need to set this
+unless you are overriding keystone's own `application_credential`
+authentication plugin.
+"""))
+
+
 GROUP_NAME = __name__.split('.')[-1]
 ALL_OPTS = [
     methods,
@@ -71,6 +93,8 @@ ALL_OPTS = [
     token,
     external,
     oauth1,
+    mapped,
+    application_credential,
 ]
 
 
