@@ -18,7 +18,6 @@ import abc
 
 import six
 
-from keystone.common import dependency
 from keystone.common import manager
 import keystone.conf
 from keystone import exception
@@ -27,11 +26,11 @@ from keystone import exception
 CONF = keystone.conf.CONF
 
 
-@dependency.provider('id_generator_api')
 class Manager(manager.Manager):
     """Default pivot point for the identifier generator backend."""
 
     driver_namespace = 'keystone.identity.id_generator'
+    _provides_api = 'id_generator_api'
 
     def __init__(self):
         super(Manager, self).__init__(CONF.identity_mapping.generator)
