@@ -15,7 +15,7 @@ from oslo_log import log
 
 from keystone.common import authorization
 from keystone.common import context
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common import tokenless_auth
 from keystone.common import wsgi
 import keystone.conf
@@ -23,7 +23,6 @@ from keystone import exception
 from keystone.federation import constants as federation_constants
 from keystone.federation import utils
 from keystone.i18n import _
-from keystone.middleware import core
 from keystone.models import token_model
 from keystone.token.providers import common
 
@@ -33,8 +32,8 @@ LOG = log.getLogger(__name__)
 __all__ = ('AuthContextMiddleware',)
 
 
-@dependency.requires('token_provider_api')
-class AuthContextMiddleware(auth_token.BaseAuthProtocol):
+class AuthContextMiddleware(provider_api.ProviderAPIMixin,
+                            auth_token.BaseAuthProtocol):
     """Build the authentication context from the request auth token."""
 
     kwargs_to_fetch_token = True
@@ -59,7 +58,7 @@ class AuthContextMiddleware(auth_token.BaseAuthProtocol):
         """
         tokenless_helper = tokenless_auth.TokenlessAuthHelper(request.environ)
 
-        (domain_id, project_id, trust_ref, unscoped) = (
+        (domain_id, project_id, trust_ref, unscoped, system) = (
             tokenless_helper.get_scope())
         user_ref = tokenless_helper.get_mapped_user(
             project_id,
@@ -136,12 +135,12 @@ class AuthContextMiddleware(auth_token.BaseAuthProtocol):
 
     @wsgi.middleware_exceptions
     def process_request(self, request):
-        context_env = request.environ.get(core.CONTEXT_ENV, {})
+        context_env = request.environ.get(wsgi.CONTEXT_ENV, {})
 
         # NOTE(notmorgan): This code is merged over from the admin token
         # middleware and now emits the security warning when the
         # conf.admin_token value is set.
-        token = request.headers.get(core.AUTH_TOKEN_HEADER)
+        token = request.headers.get(authorization.AUTH_TOKEN_HEADER)
         if CONF.admin_token and (token == CONF.admin_token):
             context_env['is_admin'] = True
             LOG.warning(
@@ -150,13 +149,15 @@ class AuthContextMiddleware(auth_token.BaseAuthProtocol):
                 "not be set. This option is deprecated in favor of using "
                 "'keystone-manage bootstrap' and will be removed in a "
                 "future release.")
-            request.environ[core.CONTEXT_ENV] = context_env
+            request.environ[wsgi.CONTEXT_ENV] = context_env
 
         if not context_env.get('is_admin', False):
             resp = super(AuthContextMiddleware, self).process_request(request)
 
             if resp:
                 return resp
+            if request.token_auth.user is not None:
+                request.set_user_headers(request.token_auth.user)
 
         # NOTE(jamielennox): function is split so testing can check errors from
         # fill_context. There is no actual reason for fill_context to raise
@@ -165,13 +166,28 @@ class AuthContextMiddleware(auth_token.BaseAuthProtocol):
         # and the middleware_exceptions helper removed.
         self.fill_context(request)
 
+    def _keystone_specific_values(self, token, request_context):
+        if token.domain_scoped:
+            # Domain scoped tokens should never have is_admin_project set
+            # Even if KSA defaults it otherwise.  The two mechanisms are
+            # parallel; only ione or the other should be used for access.
+            request_context.is_admin_project = False
+            request_context.domain_id = token.domain_id
+            request_context.domain_name = token.domain_name
+        if token.oauth_scoped:
+            request_context.is_delegated_auth = True
+            request_context.oauth_consumer_id = token.oauth_consumer_id
+            request_context.oauth_access_token_id = token.oauth_access_token_id
+        if token.trust_scoped:
+            request_context.is_delegated_auth = True
+            request_context.trust_id = token.trust_id
+        if token.is_federated_user:
+            request_context.group_ids = token.federation_group_ids
+        else:
+            request_context.group_ids = []
+
     def fill_context(self, request):
         # The request context stores itself in thread-local memory for logging.
-        request_context = context.RequestContext(
-            request_id=request.environ.get('openstack.request_id'),
-            authenticated=False,
-            overwrite=True)
-        request.environ[context.REQUEST_CONTEXT_ENV] = request_context
 
         if authorization.AUTH_CONTEXT_ENV in request.environ:
             msg = ('Auth context already exists in the request '
@@ -180,62 +196,54 @@ class AuthContextMiddleware(auth_token.BaseAuthProtocol):
             LOG.warning(msg)
             return
 
+        kwargs = {
+            'authenticated': False,
+            'overwrite': True}
+        request_context = context.RequestContext.from_environ(
+            request.environ, **kwargs)
+        request.environ[context.REQUEST_CONTEXT_ENV] = request_context
+
         # NOTE(gyee): token takes precedence over SSL client certificates.
         # This will preserve backward compatibility with the existing
         # behavior. Tokenless authorization with X.509 SSL client
         # certificate is effectively disabled if no trusted issuers are
         # provided.
 
-        if request.environ.get(core.CONTEXT_ENV, {}).get('is_admin', False):
+        if request.environ.get(wsgi.CONTEXT_ENV, {}).get('is_admin', False):
             request_context.is_admin = True
             auth_context = {}
 
         elif request.token_auth.has_user_token:
+            # Keystone enforces policy on some values that other services
+            # do not, and should not, use.  This adds them in to the context.
+            token = token_model.KeystoneToken(token_id=request.user_token,
+                                              token_data=request.token_info)
+            self._keystone_specific_values(token, request_context)
             request_context.auth_token = request.user_token
-            ref = token_model.KeystoneToken(token_id=request.user_token,
-                                            token_data=request.token_info)
-            auth_context = authorization.token_to_auth_context(ref)
+            auth_context = request_context.to_policy_values()
+            additional = {
+                'trust_id': request_context.trust_id,
+                'trustor_id': request_context.trustor_id,
+                'trustee_id': request_context.trustee_id,
+                'domain_id': request_context._domain_id,
+                'domain_name': request_context.domain_name,
+                'group_ids': request_context.group_ids,
+                'token': token
+            }
+            auth_context.update(additional)
 
         elif self._validate_trusted_issuer(request):
             auth_context = self._build_tokenless_auth_context(request)
 
         else:
-            LOG.debug('There is either no auth token in the request or '
-                      'the certificate issuer is not trusted. No auth '
-                      'context will be set.')
+            # There is either no auth token in the request or the certificate
+            # issuer is not trusted. No auth context will be set. This
+            # typically happens on an initial token request.
             return
 
         # set authenticated to flag to keystone that a token has been validated
         request_context.authenticated = True
 
-        # The attributes of request_context are put into the logs. This is a
-        # common pattern for all the OpenStack services. In all the other
-        # projects these are IDs, so set the attributes to IDs here rather than
-        # the name.
-        request_context.user_id = auth_context.get('user_id')
-        request_context.project_id = auth_context.get('project_id')
-        request_context.domain_id = auth_context.get('domain_id')
-        request_context.domain_name = auth_context.get('domain_name')
-        request_context.user_domain_id = auth_context.get('user_domain_id')
-        request_context.roles = auth_context.get('roles')
-
-        is_admin_project = auth_context.get('is_admin_project', True)
-        request_context.is_admin_project = is_admin_project
-
-        project_domain_id = auth_context.get('project_domain_id')
-        request_context.project_domain_id = project_domain_id
-
-        is_delegated_auth = auth_context.get('is_delegated_auth', False)
-        request_context.is_delegated_auth = is_delegated_auth
-
-        request_context.trust_id = auth_context.get('trust_id')
-        request_context.trustor_id = auth_context.get('trustor_id')
-        request_context.trustee_id = auth_context.get('trustee_id')
-
-        access_token_id = auth_context.get('access_token_id')
-        request_context.oauth_consumer_id = auth_context.get('consumer_id')
-        request_context.oauth_acess_token_id = access_token_id
-
         LOG.debug('RBAC: auth_context: %s', auth_context)
         request.environ[authorization.AUTH_CONTEXT_ENV] = auth_context
 
