@@ -21,9 +21,8 @@ from keystone.federation import constants
 from keystone.i18n import _
 
 # supported token versions
-V2 = 'v2.0'
 V3 = 'v3.0'
-VERSIONS = frozenset([V2, V3])
+VERSIONS = frozenset([V3])
 
 
 def _parse_and_normalize_time(time_data):
@@ -33,7 +32,7 @@ def _parse_and_normalize_time(time_data):
 
 
 class KeystoneToken(dict):
-    """An in-memory representation that unifies v2 and v3 tokens."""
+    """An in-memory representation that unifies v3 tokens."""
 
     # TODO(morganfainberg): Align this in-memory representation with the
     # objects in keystoneclient. This object should be eventually updated
@@ -48,7 +47,6 @@ class KeystoneToken(dict):
             super(KeystoneToken, self).__init__(**token_data['token'])
         except KeyError:
             raise exception.UnsupportedTokenVersionException()
-        self.token_id = token_id
 
         if self.project_scoped and self.domain_scoped:
             raise exception.UnexpectedError(_('Found invalid token: scoped to '
@@ -178,6 +176,10 @@ class KeystoneToken(dict):
             return self['is_domain']
         return False
 
+    @property
+    def system_scoped(self):
+        return 'system' in self
+
     @property
     def project_scoped(self):
         return 'project' in self
@@ -188,7 +190,7 @@ class KeystoneToken(dict):
 
     @property
     def scoped(self):
-        return self.project_scoped or self.domain_scoped
+        return self.project_scoped or self.domain_scoped or self.system_scoped
 
     @property
     def is_admin_project(self):
