@@ -18,9 +18,9 @@ import six
 from keystone import assignment
 from keystone.common import cache
 from keystone.common import clean
-from keystone.common import dependency
 from keystone.common import driver_hints
 from keystone.common import manager
+from keystone.common import provider_api
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
@@ -33,11 +33,12 @@ from keystone.token import provider as token_provider
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 MEMOIZE = cache.get_memoization_decorator(group='resource')
+PROVIDERS = provider_api.ProviderAPIs
+
+
+TAG_SEARCH_FILTERS = ('tags', 'tags-any', 'not-tags', 'not-tags-any')
 
 
-@dependency.provider('resource_api')
-@dependency.requires('assignment_api', 'credential_api', 'domain_config_api',
-                     'identity_api', 'trust_api')
 class Manager(manager.Manager):
     """Default pivot point for the Resource backend.
 
@@ -47,9 +48,11 @@ class Manager(manager.Manager):
     """
 
     driver_namespace = 'keystone.resource'
+    _provides_api = 'resource_api'
 
     _DOMAIN = 'domain'
     _PROJECT = 'project'
+    _PROJECT_TAG = 'project tag'
 
     def __init__(self):
         # NOTE(morgan): The resource driver must be SQL. This is because there
@@ -57,7 +60,9 @@ class Manager(manager.Manager):
         # SQL Identity in some form. Even if SQL Identity is not used, there
         # is almost no reason to have non-SQL Resource. Keystone requires
         # SQL in a number of ways, this simply codifies it plainly for resource
+        # the driver_name = None simply implies we don't need to load a driver.
         self.driver = resource_sql.Resource()
+        super(Manager, self).__init__(driver_name=None)
 
     def _get_hierarchy_depth(self, parents_list):
         return len(parents_list) + 1
@@ -87,7 +92,7 @@ class Manager(manager.Manager):
         :raises keystone.exception.ValidationError: If one of the constraints
             was not satisfied.
         """
-        if (not self.identity_api.multiple_domains_supported and
+        if (not PROVIDERS.identity_api.multiple_domains_supported and
                 project_ref['id'] != CONF.identity.default_domain_id):
             raise exception.ValidationError(
                 message=_('Multiple domains are not supported'))
@@ -119,19 +124,18 @@ class Manager(manager.Manager):
         if parent_ref['is_domain']:
             if parent_ref['id'] != domain['id']:
                 raise exception.ValidationError(
-                    message=_('Cannot create project, since its parent '
-                              '(%(domain_id)s) is acting as a domain, '
-                              'but project\'s specified parent_id '
-                              '(%(parent_id)s) does not match '
-                              'this domain_id.')
-                    % {'domain_id': domain['id'],
-                       'parent_id': parent_ref['id']})
+                    message=_('Cannot create project, the parent '
+                              '(%(parent_id)s) is acting as a domain, '
+                              'but this project\'s domain id (%(domain_id)s) '
+                              'does not match the parent\'s id.')
+                    % {'parent_id': parent_ref['id'],
+                       'domain_id': domain['id']})
         else:
             parent_domain_id = parent_ref.get('domain_id')
             if parent_domain_id != domain['id']:
                 raise exception.ValidationError(
                     message=_('Cannot create project, since it specifies '
-                              'its owner as domain %(domain_id)s, but '
+                              'its domain_id %(domain_id)s, but '
                               'specifies a parent in a different domain '
                               '(%(parent_domain_id)s).')
                     % {'domain_id': domain['id'],
@@ -425,7 +429,7 @@ class Manager(manager.Manager):
 
     def _pre_delete_cleanup_project(self, project_id):
         project_user_ids = (
-            self.assignment_api.list_user_ids_for_project(project_id))
+            PROVIDERS.assignment_api.list_user_ids_for_project(project_id))
         for user_id in project_user_ids:
             payload = {'user_id': user_id, 'project_id': project_id}
             notifications.Audit.internal(
@@ -439,13 +443,13 @@ class Manager(manager.Manager):
             self.get_project.invalidate(self, project_id)
             self.get_project_by_name.invalidate(self, project['name'],
                                                 project['domain_id'])
-            self.assignment_api.delete_project_assignments(project_id)
+            PROVIDERS.assignment_api.delete_project_assignments(project_id)
             # Invalidate user role assignments cache region, as it may
             # be caching role assignments where the target is
             # the specified project
             assignment.COMPUTED_ASSIGNMENTS_REGION.invalidate()
-            self.credential_api.delete_credentials_for_project(project_id)
-            self.trust_api.delete_trusts_for_project(project_id)
+            PROVIDERS.credential_api.delete_credentials_for_project(project_id)
+            PROVIDERS.trust_api.delete_trusts_for_project(project_id)
         finally:
             # attempt to send audit event even if the cache invalidation raises
             notifications.Audit.deleted(self._PROJECT, project_id, initiator)
@@ -510,7 +514,9 @@ class Manager(manager.Manager):
         return ret
 
     def _filter_projects_list(self, projects_list, user_id):
-        user_projects = self.assignment_api.list_projects_for_user(user_id)
+        user_projects = PROVIDERS.assignment_api.list_projects_for_user(
+            user_id
+        )
         user_projects_ids = set([proj['id'] for proj in user_projects])
         # Keep only the projects present in user_projects
         return [proj for proj in projects_list
@@ -767,13 +773,16 @@ class Manager(manager.Manager):
                   'first.'))
 
         self._delete_domain_contents(domain_id)
+        notifications.Audit.internal(
+            notifications.DOMAIN_DELETED, domain_id
+        )
         self._delete_project(domain_id, initiator)
         try:
             self.get_domain.invalidate(self, domain_id)
             self.get_domain_by_name.invalidate(self, domain['name'])
             # Delete any database stored domain config
-            self.domain_config_api.delete_config_options(domain_id)
-            self.domain_config_api.release_registration(domain_id)
+            PROVIDERS.domain_config_api.delete_config_options(domain_id)
+            PROVIDERS.domain_config_api.release_registration(domain_id)
         finally:
             # attempt to send audit event even if the cache invalidation raises
             notifications.Audit.deleted(self._DOMAIN, domain_id, initiator)
@@ -820,6 +829,18 @@ class Manager(manager.Manager):
 
     @manager.response_truncated
     def list_projects(self, hints=None):
+        if hints:
+            tag_filters = {}
+            # Handle project tag filters separately
+            for f in list(hints.filters):
+                if f['name'] in TAG_SEARCH_FILTERS:
+                    tag_filters[f['name']] = f['value']
+                    hints.filters.remove(f)
+            if tag_filters:
+                tag_refs = self.driver.list_projects_by_tags(tag_filters)
+                project_refs = self.driver.list_projects(hints)
+                ref_ids = [ref['id'] for ref in tag_refs]
+                return [ref for ref in project_refs if ref['id'] in ref_ids]
         return self.driver.list_projects(hints or driver_hints.Hints())
 
     # NOTE(henry-nash): list_projects_in_domain is actually an internal method
@@ -840,32 +861,6 @@ class Manager(manager.Manager):
     def get_project_by_name(self, project_name, domain_id):
         return self.driver.get_project_by_name(project_name, domain_id)
 
-    def ensure_default_domain_exists(self):
-        """Create the default domain if it doesn't exist.
-
-        This is only used for the v2 API and can go away when V2 does.
-
-        """
-        try:
-            default_domain_attrs = {
-                'name': 'Default',
-                'id': CONF.identity.default_domain_id,
-                'description': 'Domain created automatically to support V2.0 '
-                               'operations.',
-            }
-            self.create_domain(CONF.identity.default_domain_id,
-                               default_domain_attrs)
-            LOG.warning(
-                'The default domain was created automatically to contain V2 '
-                'resources. This is deprecated in the M release and will not '
-                'be supported in the O release. Create the default domain '
-                'manually or use the keystone-manage bootstrap command.')
-        except exception.Conflict:
-            LOG.debug('The default domain already exists.')
-        except Exception:
-            LOG.error('Failed to create the default domain.')
-            raise
-
     def _require_matching_domain_id(self, new_ref, orig_ref):
         """Ensure the current domain ID matches the reference one, if any.
 
@@ -882,11 +877,82 @@ class Manager(manager.Manager):
             if new_ref['domain_id'] != orig_ref['domain_id']:
                 raise exception.ValidationError(_('Cannot change Domain ID'))
 
+    def create_project_tag(self, project_id, tag, initiator=None):
+        """Create a new tag on project.
+
+        :param project_id: ID of a project to create a tag for
+        :param tag: The string value of a tag to add
+
+        :returns: The value of the created tag
+        """
+        project = self.driver.get_project(project_id)
+        tag_name = tag.strip()
+        project['tags'].append(tag_name)
+        self.update_project(project_id, {'tags': project['tags']})
+
+        notifications.Audit.created(
+            self._PROJECT_TAG, tag_name, initiator)
+        return tag_name
+
+    def get_project_tag(self, project_id, tag_name):
+        """Return information for a single tag on a project.
+
+        :param project_id: ID of a project to retrive a tag from
+        :param tag_name: Name of a tag to return
+
+        :raises keystone.exception.ProjectTagNotFound: If the tag name
+            does not exist on the project
+        :returns: The tag value
+        """
+        project = self.driver.get_project(project_id)
+        if tag_name not in project.get('tags'):
+            raise exception.ProjectTagNotFound(project_tag=tag_name)
+        return tag_name
+
+    def list_project_tags(self, project_id):
+        """List all tags on project.
+
+        :param project_id: The ID of a project
+
+        :returns: A list of tags from a project
+        """
+        project = self.driver.get_project(project_id)
+        return project.get('tags', [])
+
+    def update_project_tags(self, project_id, tags, initiator=None):
+        """Update all tags on a project.
+
+        :param project_id: The ID of the project to update
+        :param tags: A list of tags to update on the project
+
+        :returns: A list of tags
+        """
+        self.driver.get_project(project_id)
+        tag_list = [t.strip() for t in tags]
+        project = {'tags': tag_list}
+        self.update_project(project_id, project)
+        return tag_list
+
+    def delete_project_tag(self, project_id, tag):
+        """Delete single tag from project.
+
+        :param project_id: The ID of the project
+        :param tag: The tag value to delete
+
+        :raises keystone.exception.ProjectTagNotFound: If the tag name
+            does not exist on the project
+        """
+        project = self.driver.get_project(project_id)
+        try:
+            project['tags'].remove(tag)
+        except ValueError:
+            raise exception.ProjectTagNotFound(project_tag=tag)
+        self.update_project(project_id, project)
+        notifications.Audit.deleted(self._PROJECT_TAG, tag)
 
 MEMOIZE_CONFIG = cache.get_memoization_decorator(group='domain_config')
 
 
-@dependency.provider('domain_config_api')
 class DomainConfigManager(manager.Manager):
     """Default pivot point for the Domain Config backend."""
 
@@ -900,6 +966,7 @@ class DomainConfigManager(manager.Manager):
     # the identity manager are supported.
 
     driver_namespace = 'keystone.resource.domain_config'
+    _provides_api = 'domain_config_api'
 
     # We explicitly state each whitelisted option instead of pulling all ldap
     # options from CONF and selectively pruning them to prevent a security
