@@ -14,7 +14,6 @@
 
 from oslo_log import log
 
-from keystone.common import dependency
 from keystone.common import sql
 import keystone.conf
 from keystone.server import backends
@@ -47,5 +46,4 @@ def setup_backends(load_extra_backends_fn=lambda: {},
     drivers = backends.load_backends()
     drivers.update(load_extra_backends_fn())
     res = startup_application_fn()
-    drivers.update(dependency.resolve_future_dependencies())
     return drivers, res
