@@ -14,14 +14,18 @@
 class AuthTestMixin(object):
     """To hold auth building helper functions."""
 
-    def _build_auth_scope(self, project_id=None, project_name=None,
-                          project_domain_id=None, project_domain_name=None,
-                          domain_id=None, domain_name=None, trust_id=None,
-                          unscoped=None):
+    def _build_auth_scope(self, system=False, project_id=None,
+                          project_name=None, project_domain_id=None,
+                          project_domain_name=None, domain_id=None,
+                          domain_name=None, trust_id=None, unscoped=None):
         scope_data = {}
-        if unscoped:
+        if system:
+            scope_data['system'] = {'all': True}
+        elif unscoped:
             scope_data['unscoped'] = {}
-        if project_id or project_name:
+        elif system:
+            scope_data['system'] = {'all': True}
+        elif project_id or project_name:
             scope_data['project'] = {}
             if project_id:
                 scope_data['project']['id'] = project_id
@@ -34,17 +38,36 @@ class AuthTestMixin(object):
                     else:
                         project_domain_json['name'] = project_domain_name
                     scope_data['project']['domain'] = project_domain_json
-        if domain_id or domain_name:
+        elif domain_id or domain_name:
             scope_data['domain'] = {}
             if domain_id:
                 scope_data['domain']['id'] = domain_id
             else:
                 scope_data['domain']['name'] = domain_name
-        if trust_id:
+        elif trust_id:
             scope_data['OS-TRUST:trust'] = {}
             scope_data['OS-TRUST:trust']['id'] = trust_id
+        else:
+            raise ValueError(_('Programming Error: Invalid arguments supplied '
+                               'to build scope.'))
+
         return scope_data
 
+    def _build_user(self, user_id=None, username=None, user_domain_id=None,
+                    user_domain_name=None):
+        user = {}
+        if user_id:
+            user['id'] = user_id
+        else:
+            user['name'] = username
+            if user_domain_id or user_domain_name:
+                user['domain'] = {}
+                if user_domain_id:
+                    user['domain']['id'] = user_domain_id
+                else:
+                    user['domain']['name'] = user_domain_name
+        return user
+
     def _build_auth(self, user_id=None, username=None, user_domain_id=None,
                     user_domain_name=None, **kwargs):
 
@@ -60,27 +83,36 @@ class AuthTestMixin(object):
                       message="_build_auth only supports 'passcode' "
                               "and 'password' secret types")
 
-        data = {'user': {}}
-        if user_id:
-            data['user']['id'] = user_id
-        else:
-            data['user']['name'] = username
-            if user_domain_id or user_domain_name:
-                data['user']['domain'] = {}
-                if user_domain_id:
-                    data['user']['domain']['id'] = user_domain_id
-                else:
-                    data['user']['domain']['name'] = user_domain_name
+        data = {}
+        data['user'] = self._build_user(user_id=user_id, username=username,
+                                        user_domain_id=user_domain_id,
+                                        user_domain_name=user_domain_name)
         data['user'][secret_type] = secret_value
         return data
 
     def _build_token_auth(self, token):
         return {'id': token}
 
+    def _build_app_cred_auth(self, secret, app_cred_id=None,
+                             app_cred_name=None, user_id=None, username=None,
+                             user_domain_id=None, user_domain_name=None):
+        data = {'secret': secret}
+        if app_cred_id:
+            data['id'] = app_cred_id
+        else:
+            data['name'] = app_cred_name
+            data['user'] = self._build_user(user_id=user_id,
+                                            username=username,
+                                            user_domain_id=user_domain_id,
+                                            user_domain_name=user_domain_name)
+        return data
+
     def build_authentication_request(self, token=None, user_id=None,
                                      username=None, user_domain_id=None,
                                      user_domain_name=None, password=None,
-                                     kerberos=False, passcode=None, **kwargs):
+                                     kerberos=False, passcode=None,
+                                     app_cred_id=None, app_cred_name=None,
+                                     secret=None, **kwargs):
         """Build auth dictionary.
 
         It will create an auth dictionary based on all the arguments
@@ -104,6 +136,14 @@ class AuthTestMixin(object):
             auth_data['identity']['totp'] = self._build_auth(
                 user_id, username, user_domain_id, user_domain_name,
                 passcode=passcode)
+        if (app_cred_id or app_cred_name) and secret:
+            auth_data['identity']['methods'].append('application_credential')
+            identity = auth_data['identity']
+            identity['application_credential'] = self._build_app_cred_auth(
+                secret, app_cred_id=app_cred_id, app_cred_name=app_cred_name,
+                user_id=user_id, username=username,
+                user_domain_id=user_domain_id,
+                user_domain_name=user_domain_name)
         if kwargs:
             auth_data['scope'] = self._build_auth_scope(**kwargs)
         return {'auth': auth_data}
