@@ -20,7 +20,7 @@ from oslo_config import fixture as config_fixture
 from oslo_log import log
 import six
 
-from keystone.common import fernet_utils
+from keystone.common import token_utils
 from keystone.common import utils as common_utils
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
@@ -71,6 +71,34 @@ class UtilsTestCase(unit.BaseTestCase):
         self.assertRaises(ValueError, common_utils.resource_uuid,
                           value)
 
+    def test_remove_duplicate_dicts_from_list(self):
+        dict_list = []
+        num_of_duplicate = 10
+        dup_value = {
+            'id': uuid.uuid4().hex,
+            'name': uuid.uuid4().hex,
+            'tags': ['foo', 'bar']
+        }
+        # Add in 10 unique items, and 10 of the same item
+        for i in range(num_of_duplicate):
+            new_dict = {
+                'id': i,
+                'name': uuid.uuid4().hex,
+                'tags': ['foo', i]
+            }
+            dict_list.append(new_dict)
+            dict_list.append(dup_value)
+        self.assertEqual(len(dict_list), 20)
+        result = common_utils.remove_duplicate_dicts_by_id(dict_list)
+        # Assert that 9 duplicate items have been removed
+        self.assertEqual(len(result), len(dict_list) - 9)
+        # Show that the duplicate item is only stored once
+        count = 0
+        for r in result:
+            if r['id'] == dup_value['id']:
+                count += 1
+        self.assertEqual(count, 1)
+
     def test_hash(self):
         password = 'right'
         wrong = 'wrongwrong'  # Two wrongs don't make a right
@@ -244,10 +272,10 @@ class ServiceHelperTests(unit.BaseTestCase):
         self.assertRaises(unit.UnexpectedExit, self._do_test)
 
 
-class FernetUtilsTestCase(unit.BaseTestCase):
+class TokenUtilsTestCase(unit.BaseTestCase):
 
     def setUp(self):
-        super(FernetUtilsTestCase, self).setUp()
+        super(TokenUtilsTestCase, self).setUp()
         self.config_fixture = self.useFixture(config_fixture.Config(CONF))
 
     def test_debug_message_logged_when_loading_fernet_token_keys(self):
@@ -259,7 +287,7 @@ class FernetUtilsTestCase(unit.BaseTestCase):
             )
         )
         logging_fixture = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
-        fernet_utilities = fernet_utils.FernetUtils(
+        fernet_utilities = token_utils.TokenUtils(
             CONF.fernet_tokens.key_repository,
             CONF.fernet_tokens.max_active_keys,
             'fernet_tokens'
@@ -282,7 +310,7 @@ class FernetUtilsTestCase(unit.BaseTestCase):
             )
         )
         logging_fixture = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
-        fernet_utilities = fernet_utils.FernetUtils(
+        fernet_utilities = token_utils.TokenUtils(
             CONF.credential.key_repository,
             credential_fernet.MAX_ACTIVE_KEYS,
             'credential'
