@@ -16,15 +16,12 @@ import uuid
 
 from oslo_log import log
 
-from keystone.common import fernet_utils
-import keystone.conf
+from keystone.common import token_utils
 from keystone.credential.providers import fernet as credential_fernet
 from keystone.tests import unit
 from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 
-CONF = keystone.conf.CONF
-
 
 class TestFernetCredentialProvider(unit.TestCase):
     def setUp(self):
@@ -66,8 +63,12 @@ class TestFernetCredentialProviderWithNullKey(unit.TestCase):
         )
 
     def test_encryption_with_null_key(self):
-        null_key = fernet_utils.NULL_KEY
-        null_key_hash = hashlib.sha1(null_key).hexdigest()
+        null_key = token_utils.NULL_KEY
+        # NOTE(lhinds) This is marked as #nosec since bandit will see SHA1
+        # which is marked insecure. Keystone uses SHA1 in this case as part of
+        # HMAC-SHA1 which is currently not insecure but will still get
+        # caught when scanning with bandit.
+        null_key_hash = hashlib.sha1(null_key).hexdigest()  # nosec
 
         blob = uuid.uuid4().hex
         encrypted_blob, primary_key_hash = self.provider.encrypt(blob)
