@@ -21,6 +21,9 @@ BAZ_TENANT_ID = uuid.uuid4().hex
 MTU_TENANT_ID = uuid.uuid4().hex
 SERVICE_TENANT_ID = uuid.uuid4().hex
 DEFAULT_DOMAIN_ID = 'default'
+ADMIN_ROLE_ID = uuid.uuid4().hex
+MEMBER_ROLE_ID = uuid.uuid4().hex
+OTHER_ROLE_ID = uuid.uuid4().hex
 
 TENANTS = [
     {
@@ -31,6 +34,7 @@ TENANTS = [
         'enabled': True,
         'parent_id': DEFAULT_DOMAIN_ID,
         'is_domain': False,
+        'tags': []
     }, {
         'id': BAZ_TENANT_ID,
         'name': 'BAZ',
@@ -39,6 +43,7 @@ TENANTS = [
         'enabled': True,
         'parent_id': DEFAULT_DOMAIN_ID,
         'is_domain': False,
+        'tags': []
     }, {
         'id': MTU_TENANT_ID,
         'name': 'MTU',
@@ -47,6 +52,7 @@ TENANTS = [
         'domain_id': DEFAULT_DOMAIN_ID,
         'parent_id': DEFAULT_DOMAIN_ID,
         'is_domain': False,
+        'tags': []
     }, {
         'id': SERVICE_TENANT_ID,
         'name': 'service',
@@ -55,6 +61,7 @@ TENANTS = [
         'domain_id': DEFAULT_DOMAIN_ID,
         'parent_id': DEFAULT_DOMAIN_ID,
         'is_domain': False,
+        'tags': []
     }
 ]
 
@@ -62,8 +69,8 @@ TENANTS = [
 USERS = [
     # NOTE(morganfainberg): Admin user for replacing admin_token_auth
     {
-        'id': 'reqadmin',
-        'name': 'REQ_ADMIN',
+        'id': uuid.uuid4().hex,
+        'name': 'req_admin',
         'domain_id': DEFAULT_DOMAIN_ID,
         'password': 'password',
         'tenants': [],
@@ -71,8 +78,8 @@ USERS = [
         'options': {},
     },
     {
-        'id': 'foo',
-        'name': 'FOO',
+        'id': uuid.uuid4().hex,
+        'name': 'foo',
         'domain_id': DEFAULT_DOMAIN_ID,
         'password': 'foo2',
         'tenants': [BAR_TENANT_ID],
@@ -80,8 +87,8 @@ USERS = [
         'email': 'foo@bar.com',
         'options': {},
     }, {
-        'id': 'two',
-        'name': 'TWO',
+        'id': uuid.uuid4().hex,
+        'name': 'two',
         'domain_id': DEFAULT_DOMAIN_ID,
         'password': 'two2',
         'enabled': True,
@@ -90,8 +97,8 @@ USERS = [
         'email': 'two@three.com',
         'options': {},
     }, {
-        'id': 'badguy',
-        'name': 'BadGuy',
+        'id': uuid.uuid4().hex,
+        'name': 'badguy',
         'domain_id': DEFAULT_DOMAIN_ID,
         'password': 'bad',
         'enabled': False,
@@ -100,8 +107,8 @@ USERS = [
         'email': 'bad@guy.com',
         'options': {},
     }, {
-        'id': 'sna',
-        'name': 'SNA',
+        'id': uuid.uuid4().hex,
+        'name': 'sna',
         'domain_id': DEFAULT_DOMAIN_ID,
         'password': 'snafu',
         'enabled': True,
@@ -113,32 +120,32 @@ USERS = [
 
 ROLES = [
     {
-        'id': 'admin',
+        'id': ADMIN_ROLE_ID,
         'name': 'admin',
         'domain_id': None,
     }, {
-        'id': 'member',
-        'name': 'Member',
+        'id': MEMBER_ROLE_ID,
+        'name': 'member',
         'domain_id': None,
     }, {
         'id': '9fe2ff9ee4384b1894a90878d3e92bab',
         'name': '_member_',
         'domain_id': None,
     }, {
-        'id': 'other',
-        'name': 'Other',
+        'id': OTHER_ROLE_ID,
+        'name': 'other',
         'domain_id': None,
     }, {
-        'id': 'browser',
-        'name': 'Browser',
+        'id': uuid.uuid4().hex,
+        'name': 'browser',
         'domain_id': None,
     }, {
-        'id': 'writer',
-        'name': 'Writer',
+        'id': uuid.uuid4().hex,
+        'name': 'writer',
         'domain_id': None,
     }, {
-        'id': 'service',
-        'name': 'Service',
+        'id': uuid.uuid4().hex,
+        'name': 'service',
         'domain_id': None,
     }
 ]
@@ -146,9 +153,9 @@ ROLES = [
 # NOTE(morganfainberg): Admin assignment for replacing admin_token_auth
 ROLE_ASSIGNMENTS = [
     {
-        'user': 'reqadmin',
+        'user': 'req_admin',
         'tenant_id': SERVICE_TENANT_ID,
-        'role_id': 'admin'
+        'role_id': ADMIN_ROLE_ID
     },
 ]
 
@@ -157,3 +164,12 @@ DOMAINS = [{'description':
             'enabled': True,
             'id': DEFAULT_DOMAIN_ID,
             'name': u'Default'}]
+
+SERVICES = [{'id': uuid.uuid4().hex,
+             'type': 'type_one',
+             'enabled': True,
+             'extra': {'description': 'This is a service for test.',
+                       'name': 'service_one'}
+             }]
+
+REGIONS = [{'id': 'region_one'}, {'id': 'region_two'}]
