@@ -27,14 +27,13 @@ import re
 import shelve
 
 import ldap
-from oslo_config import cfg
 from oslo_log import log
 import six
 from six import moves
 
-from keystone.common.ldap import core
+import keystone.conf
 from keystone import exception
-
+from keystone.identity.backends.ldap import common
 
 SCOPE_NAMES = {
     ldap.SCOPE_BASE: 'SCOPE_BASE',
@@ -42,16 +41,13 @@ SCOPE_NAMES = {
     ldap.SCOPE_SUBTREE: 'SCOPE_SUBTREE',
 }
 
-# http://msdn.microsoft.com/en-us/library/windows/desktop/aa366991(v=vs.85).aspx  # noqa
-CONTROL_TREEDELETE = '1.2.840.113556.1.4.805'
-
 LOG = log.getLogger(__name__)
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 
 
 def _internal_attr(attr_name, value_or_values):
     def normalize_value(value):
-        return core.utf8_decode(value)
+        return common.utf8_decode(value)
 
     def normalize_dn(dn):
         # Capitalize the attribute names as an LDAP server might.
@@ -69,7 +65,7 @@ def _internal_attr(attr_name, value_or_values):
             return 'CN=Doe\\2C John,OU=Users,CN=example,CN=com'
 
         try:
-            dn = ldap.dn.str2dn(core.utf8_encode(dn))
+            dn = ldap.dn.str2dn(common.utf8_encode(dn))
         except ldap.DECODING_ERROR:
             # NOTE(amakarov): In case of IDs instead of DNs in group members
             # they must be handled as regular values.
@@ -78,11 +74,10 @@ def _internal_attr(attr_name, value_or_values):
         norm = []
         for part in dn:
             name, val, i = part[0]
-            name = core.utf8_decode(name)
+            name = common.utf8_decode(name)
             name = name.upper()
-            name = core.utf8_encode(name)
             norm.append([(name, val, i)])
-        return core.utf8_decode(ldap.dn.dn2str(norm))
+        return common.utf8_decode(ldap.dn.dn2str(norm))
 
     if attr_name in ('member', 'roleOccupant'):
         attr_fn = normalize_dn
@@ -171,9 +166,9 @@ def _match(key, value, attrs):
     if value == '*':
         return True
     if key == 'serviceId':
-        # for serviceId, the backend is returning a list of numbers
-        # make sure we convert them to strings first before comparing
-        # them
+        # For serviceId, the backend is returning a list of numbers.
+        # Make sure we convert them to strings first before comparing
+        # them.
         str_sids = [six.text_type(x) for x in attrs[key]]
         return six.text_type(value) in str_sids
     if key != 'objectclass':
@@ -181,7 +176,7 @@ def _match(key, value, attrs):
         norm_values = list(
             _internal_attr(key, x)[0].lower() for x in attrs[key])
         return match_with_wildcards(check_value, norm_values)
-    # it is an objectclass check, so check subclasses
+    # It is an objectclass check, so check subclasses
     values = _subs(value)
     for v in values:
         if v in attrs[key]:
@@ -190,7 +185,7 @@ def _match(key, value, attrs):
 
 
 def _subs(value):
-    """Returns a list of subclass strings.
+    """Return a list of subclass strings.
 
     The strings represent the ldap objectclass plus any subclasses that
     inherit from it. Fakeldap doesn't know about the ldap object structure,
@@ -218,7 +213,7 @@ FakeShelves = {}
 PendingRequests = {}
 
 
-class FakeLdap(core.LDAPHandler):
+class FakeLdap(common.LDAPHandler):
     """Emulate the python-ldap API.
 
     The python-ldap API requires all strings to be UTF-8 encoded. This
@@ -246,7 +241,8 @@ class FakeLdap(core.LDAPHandler):
                 tls_req_cert='demand', chase_referrals=None, debug_level=None,
                 use_pool=None, pool_size=None, pool_retry_max=None,
                 pool_retry_delay=None, pool_conn_timeout=None,
-                pool_conn_lifetime=None):
+                pool_conn_lifetime=None,
+                conn_timeout=None):
         if url.startswith('fake://memory'):
             if url not in FakeShelves:
                 FakeShelves[url] = FakeShelve()
@@ -264,7 +260,7 @@ class FakeLdap(core.LDAPHandler):
                 ldap.set_option(ldap.OPT_X_TLS_CACERTFILE, tls_cacertfile)
             elif tls_cacertdir:
                 ldap.set_option(ldap.OPT_X_TLS_CACERTDIR, tls_cacertdir)
-            if tls_req_cert in list(core.LDAP_TLS_CERTS.values()):
+            if tls_req_cert in list(common.LDAP_TLS_CERTS.values()):
                 ldap.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, tls_req_cert)
             else:
                 raise ValueError("invalid TLS_REQUIRE_CERT tls_req_cert=%s",
@@ -280,49 +276,53 @@ class FakeLdap(core.LDAPHandler):
         self.pool_retry_delay = pool_retry_delay
         self.pool_conn_timeout = pool_conn_timeout
         self.pool_conn_lifetime = pool_conn_lifetime
+        self.conn_timeout = conn_timeout
 
     def dn(self, dn):
-        return core.utf8_decode(dn)
+        return common.utf8_decode(dn)
 
     def _dn_to_id_attr(self, dn):
-        return core.utf8_decode(ldap.dn.str2dn(core.utf8_encode(dn))[0][0][0])
+        return common.utf8_decode(
+            ldap.dn.str2dn(common.utf8_encode(dn))[0][0][0])
 
     def _dn_to_id_value(self, dn):
-        return core.utf8_decode(ldap.dn.str2dn(core.utf8_encode(dn))[0][0][1])
+        return common.utf8_decode(
+            ldap.dn.str2dn(common.utf8_encode(dn))[0][0][1])
 
     def key(self, dn):
         return '%s%s' % (self.__prefix, self.dn(dn))
 
     def simple_bind_s(self, who='', cred='',
                       serverctrls=None, clientctrls=None):
-        """This method is ignored, but provided for compatibility."""
+        """Provide for compatibility but this method is ignored."""
         if server_fail:
             raise ldap.SERVER_DOWN
         whos = ['cn=Admin', CONF.ldap.user]
-        if who in whos and cred in ['password', CONF.ldap.password]:
+        if (common.utf8_decode(who) in whos and
+                common.utf8_decode(cred) in ['password', CONF.ldap.password]):
             return
 
-        try:
-            attrs = self.db[self.key(who)]
-        except KeyError:
-            LOG.debug('bind fail: who=%s not found', core.utf8_decode(who))
-            raise ldap.NO_SUCH_OBJECT
+        attrs = self.db.get(self.key(who))
+        if not attrs:
+            LOG.debug('who=%s not found, binding anonymously',
+                      common.utf8_decode(who))
 
-        db_password = None
-        try:
-            db_password = attrs['userPassword'][0]
-        except (KeyError, IndexError):
-            LOG.debug('bind fail: password for who=%s not found',
-                      core.utf8_decode(who))
-            raise ldap.INAPPROPRIATE_AUTH
+        db_password = ''
+        if attrs:
+            try:
+                db_password = attrs['userPassword'][0]
+            except (KeyError, IndexError):
+                LOG.debug('bind fail: password for who=%s not found',
+                          common.utf8_decode(who))
+                raise ldap.INAPPROPRIATE_AUTH
 
-        if cred != db_password:
+        if cred != common.utf8_encode(db_password):
             LOG.debug('bind fail: password for who=%s does not match',
-                      core.utf8_decode(who))
+                      common.utf8_decode(who))
             raise ldap.INVALID_CREDENTIALS
 
     def unbind_s(self):
-        """This method is ignored, but provided for compatibility."""
+        """Provide for compatibility but this method is ignored."""
         if server_fail:
             raise ldap.SERVER_DOWN
 
@@ -343,19 +343,19 @@ class FakeLdap(core.LDAPHandler):
 
             if k == id_attr:
                 for val in dummy_v:
-                    if core.utf8_decode(val) == id_value:
+                    if common.utf8_decode(val) == id_value:
                         id_attr_in_modlist = True
 
         if not id_attr_in_modlist:
-            LOG.debug('id_attribute=%(attr)s missing, attributes=%(attrs)s' %
+            LOG.debug('id_attribute=%(attr)s missing, attributes=%(attrs)s',
                       {'attr': id_attr, 'attrs': modlist})
             raise ldap.NAMING_VIOLATION
         key = self.key(dn)
         LOG.debug('add item: dn=%(dn)s, attrs=%(attrs)s', {
-            'dn': core.utf8_decode(dn), 'attrs': modlist})
+            'dn': common.utf8_decode(dn), 'attrs': modlist})
         if key in self.db:
             LOG.debug('add item failed: dn=%s is already in store.',
-                      core.utf8_decode(dn))
+                      common.utf8_decode(dn))
             raise ldap.ALREADY_EXISTS(dn)
 
         self.db[key] = {k: _internal_attr(k, v) for k, v in modlist}
@@ -377,19 +377,12 @@ class FakeLdap(core.LDAPHandler):
             raise ldap.SERVER_DOWN
 
         try:
-            if CONTROL_TREEDELETE in [c.controlType for c in serverctrls]:
-                LOG.debug('FakeLdap subtree_delete item: dn=%s',
-                          core.utf8_decode(dn))
-                children = self._getChildren(dn)
-                for c in children:
-                    del self.db[c]
-
             key = self.key(dn)
-            LOG.debug('FakeLdap delete item: dn=%s', core.utf8_decode(dn))
+            LOG.debug('FakeLdap delete item: dn=%s', common.utf8_decode(dn))
             del self.db[key]
         except KeyError:
             LOG.debug('delete item failed: dn=%s not found.',
-                      core.utf8_decode(dn))
+                      common.utf8_decode(dn))
             raise ldap.NO_SUCH_OBJECT
         self.db.sync()
 
@@ -405,12 +398,12 @@ class FakeLdap(core.LDAPHandler):
 
         key = self.key(dn)
         LOG.debug('modify item: dn=%(dn)s attrs=%(attrs)s', {
-            'dn': core.utf8_decode(dn), 'attrs': modlist})
+            'dn': common.utf8_decode(dn), 'attrs': modlist})
         try:
             entry = self.db[key]
         except KeyError:
             LOG.debug('modify item failed: dn=%s not found.',
-                      core.utf8_decode(dn))
+                      common.utf8_decode(dn))
             raise ldap.NO_SUCH_OBJECT
 
         for cmd, k, v in modlist:
@@ -474,9 +467,9 @@ class FakeLdap(core.LDAPHandler):
             results = [(base, item_dict)]
         elif scope == ldap.SCOPE_SUBTREE:
             # FIXME - LDAP search with SUBTREE scope must return the base
-            # entry, but the code below does _not_.  Unfortunately, there are
+            # entry, but the code below does _not_. Unfortunately, there are
             # several tests that depend on this broken behavior, and fail
-            # when the base entry is returned in the search results.  The
+            # when the base entry is returned in the search results. The
             # fix is easy here, just initialize results as above for
             # the SCOPE_BASE case.
             # https://bugs.launchpad.net/keystone/+bug/1368772
@@ -495,14 +488,14 @@ class FakeLdap(core.LDAPHandler):
         elif scope == ldap.SCOPE_ONELEVEL:
 
             def get_entries():
-                base_dn = ldap.dn.str2dn(core.utf8_encode(base))
+                base_dn = ldap.dn.str2dn(common.utf8_encode(base))
                 base_len = len(base_dn)
 
                 for k, v in self.db.items():
                     if not k.startswith(self.__prefix):
                         continue
                     k_dn_str = k[len(self.__prefix):]
-                    k_dn = ldap.dn.str2dn(core.utf8_encode(k_dn_str))
+                    k_dn = ldap.dn.str2dn(common.utf8_encode(k_dn_str))
                     if len(k_dn) != base_len + 1:
                         continue
                     if k_dn[-base_len:] != base_dn:
@@ -518,13 +511,14 @@ class FakeLdap(core.LDAPHandler):
         objects = []
         for dn, attrs in results:
             # filter the objects by filterstr
-            id_attr, id_val, _ = ldap.dn.str2dn(core.utf8_encode(dn))[0][0]
-            id_attr = core.utf8_decode(id_attr)
-            id_val = core.utf8_decode(id_val)
+            id_attr, id_val, _ = ldap.dn.str2dn(common.utf8_encode(dn))[0][0]
+            id_attr = common.utf8_decode(id_attr)
+            id_val = common.utf8_decode(id_val)
             match_attrs = attrs.copy()
             match_attrs[id_attr] = [id_val]
             attrs_checked = set()
-            if not filterstr or _match_query(filterstr, match_attrs,
+            if not filterstr or _match_query(common.utf8_decode(filterstr),
+                                             match_attrs,
                                              attrs_checked):
                 if (filterstr and
                         (scope != ldap.SCOPE_BASE) and
@@ -532,7 +526,7 @@ class FakeLdap(core.LDAPHandler):
                     raise AssertionError('No objectClass in search filter')
                 # filter the attributes by attrlist
                 attrs = {k: v for k, v in attrs.items()
-                         if not attrlist or k in attrlist}
+                         if not attrlist or k in common.utf8_decode(attrlist)}
                 objects.append((dn, attrs))
 
         return objects
@@ -566,7 +560,7 @@ class FakeLdap(core.LDAPHandler):
 
     def result3(self, msgid=ldap.RES_ANY, all=1, timeout=None,
                 resp_ctrl_classes=None):
-        """Execute async request
+        """Execute async request.
 
         Only msgid param is supported. Request info is fetched from global
         variable `PendingRequests` by msgid, executed using search_s and
@@ -638,7 +632,7 @@ class FakeLdapPool(FakeLdap):
 
 
 class FakeLdapNoSubtreeDelete(FakeLdap):
-    """FakeLdap subclass that does not support subtree delete
+    """FakeLdap subclass that does not support subtree delete.
 
     Same as FakeLdap except delete will throw the LDAP error
     ldap.NOT_ALLOWED_ON_NONLEAF if there is an attempt to delete
@@ -657,7 +651,7 @@ class FakeLdapNoSubtreeDelete(FakeLdap):
 
         except KeyError:
             LOG.debug('delete item failed: dn=%s not found.',
-                      core.utf8_decode(dn))
+                      common.utf8_decode(dn))
             raise ldap.NO_SUCH_OBJECT
         super(FakeLdapNoSubtreeDelete, self).delete_ext_s(dn,
                                                           serverctrls,
