@@ -14,7 +14,6 @@
 import fixtures
 
 from keystone import auth
-from keystone.common import dependency
 from keystone.server import common
 
 
@@ -28,11 +27,6 @@ class BackendLoader(fixtures.Fixture):
     def setUp(self):
         super(BackendLoader, self).setUp()
 
-        # TODO(blk-u): Shouldn't need to clear the registry here, but some
-        # tests call load_backends multiple times. These should be fixed to
-        # only call load_backends once.
-        dependency.reset()
-
         self.clear_auth_plugin_registry()
         drivers, _unused = common.setup_backends()
 
