@@ -16,9 +16,9 @@ from oslo_serialization import jsonutils
 from six.moves import http_client
 import webtest
 
-from keystone.auth import controllers as auth_controllers
 from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
+from keystone.tests.unit import ksfixtures
 from keystone.tests.unit.ksfixtures import database
 
 
@@ -52,16 +52,12 @@ class RestfulTestCase(unit.TestCase):
     # default content type to test
     content_type = 'json'
 
-    def get_extensions(self):
-        return None
-
     def setUp(self, app_conf='keystone'):
         super(RestfulTestCase, self).setUp()
 
-        # Will need to reset the plug-ins
-        self.addCleanup(setattr, auth_controllers, 'AUTH_METHODS', {})
+        self.auth_plugin_config_override()
 
-        self.useFixture(database.Database(self.sql_driver_version_overrides))
+        self.useFixture(database.Database())
         self.load_backends()
         self.load_fixtures(default_fixtures)
 
@@ -72,6 +68,12 @@ class RestfulTestCase(unit.TestCase):
             self.loadapp(app_conf, name='admin'))
         self.addCleanup(delattr, self, 'admin_app')
 
+    def auth_plugin_config_override(self, methods=None, **method_classes):
+        self.useFixture(
+            ksfixtures.ConfigAuthPlugins(self.config_fixture,
+                                         methods,
+                                         **method_classes))
+
     def request(self, app, path, body=None, headers=None, token=None,
                 expected_status=None, **kwargs):
         if headers:
@@ -92,7 +94,7 @@ class RestfulTestCase(unit.TestCase):
         return response
 
     def assertResponseSuccessful(self, response):
-        """Asserts that a status code lies inside the 2xx range.
+        """Assert that a status code lies inside the 2xx range.
 
         :param response: :py:class:`httplib.HTTPResponse` to be
           verified to have a status code between 200 and 299.
@@ -102,12 +104,12 @@ class RestfulTestCase(unit.TestCase):
              self.assertResponseSuccessful(response)
         """
         self.assertTrue(
-            response.status_code >= 200 and response.status_code <= 299,
+            200 <= response.status_code <= 299,
             'Status code %d is outside of the expected range (2xx)\n\n%s' %
             (response.status, response.body))
 
     def assertResponseStatus(self, response, expected_status):
-        """Asserts a specific status code on the response.
+        """Assert a specific status code on the response.
 
         :param response: :py:class:`httplib.HTTPResponse`
         :param expected_status: The specific ``status`` result expected
@@ -122,7 +124,7 @@ class RestfulTestCase(unit.TestCase):
             (response.status_code, expected_status, response.body))
 
     def assertValidResponseHeaders(self, response):
-        """Ensures that response headers appear as expected."""
+        """Ensure that response headers appear as expected."""
         self.assertIn('X-Auth-Token', response.headers.get('Vary'))
 
     def assertValidErrorResponse(self, response,
@@ -168,7 +170,7 @@ class RestfulTestCase(unit.TestCase):
     def restful_request(self, method='GET', headers=None, body=None,
                         content_type=None, response_content_type=None,
                         **kwargs):
-        """Serializes/deserializes json as request/response body.
+        """Serialize/deserialize json as request/response body.
 
         .. WARNING::
 
@@ -220,8 +222,8 @@ class RestfulTestCase(unit.TestCase):
         return self._get_token({
             'auth': {
                 'passwordCredentials': {
-                    'username': self.user_reqadmin['name'],
-                    'password': self.user_reqadmin['password']
+                    'username': self.user_req_admin['name'],
+                    'password': self.user_req_admin['password']
                 },
                 'tenantId': default_fixtures.SERVICE_TENANT_ID
             }
