@@ -16,6 +16,7 @@ import logging
 import os
 import uuid
 
+import argparse
 import fixtures
 import mock
 import oslo_config.fixture
@@ -36,7 +37,7 @@ from keystone.cmd.doctor import ldap
 from keystone.cmd.doctor import security_compliance
 from keystone.cmd.doctor import tokens
 from keystone.cmd.doctor import tokens_fernet
-from keystone.common import dependency
+from keystone.common import provider_api
 from keystone.common.sql import upgrades
 import keystone.conf
 from keystone import exception
@@ -49,6 +50,7 @@ from keystone.tests.unit.ksfixtures import ldapdb
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class CliTestCase(unit.SQLDriverOverrides, unit.TestCase):
@@ -60,6 +62,11 @@ class CliTestCase(unit.SQLDriverOverrides, unit.TestCase):
     def test_token_flush(self):
         self.useFixture(database.Database())
         self.load_backends()
+        # NOTE(morgan): we are testing a direct instantiation of the
+        # persistence manager for flushing. We should clear this out so we
+        # don't error. CLI should never impact a running service
+        # and should never actually lock the registry for dependencies.
+        provider_api.ProviderAPIs._clear_registry_instances()
         cli.TokenFlush.main()
 
 
@@ -124,6 +131,13 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 project['id']))
         self.assertIs(1, len(role_list))
         self.assertEqual(role_list[0], role['id'])
+        system_roles = (
+            bootstrap.assignment_manager.list_system_grants_for_user(
+                user['id']
+            )
+        )
+        self.assertIs(1, len(system_roles))
+        self.assertEqual(system_roles[0]['id'], role['id'])
         # NOTE(morganfainberg): Pass an empty context, it isn't used by
         # `authenticate` method.
         bootstrap.identity_manager.authenticate(
@@ -246,19 +260,6 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
             user_id,
             bootstrap.password)
 
-    def test_bootstrap_creates_default_role(self):
-        bootstrap = cli.BootStrap()
-        try:
-            role = bootstrap.role_manager.get_role(CONF.member_role_id)
-            self.fail('Member Role is created and should not be.')
-        except exception.RoleNotFound:
-            pass
-
-        self._do_test_bootstrap(bootstrap)
-        role = bootstrap.role_manager.get_role(CONF.member_role_id)
-        self.assertEqual(role['name'], CONF.member_role_name)
-        self.assertEqual(role['id'], CONF.member_role_id)
-
 
 class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
 
@@ -403,13 +404,15 @@ class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
             if domain == 'domain_default':
                 # Not allowed to delete the default domain, but should at least
                 # delete any domain-specific config for it.
-                self.domain_config_api.delete_config(
+                PROVIDERS.domain_config_api.delete_config(
                     CONF.identity.default_domain_id)
                 continue
             this_domain = self.domains[domain]
             this_domain['enabled'] = False
-            self.resource_api.update_domain(this_domain['id'], this_domain)
-            self.resource_api.delete_domain(this_domain['id'])
+            PROVIDERS.resource_api.update_domain(
+                this_domain['id'], this_domain
+            )
+            PROVIDERS.resource_api.delete_domain(this_domain['id'])
         self.domains = {}
 
     def config(self, config_files):
@@ -419,7 +422,7 @@ class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
     def setup_initial_domains(self):
 
         def create_domain(domain):
-            return self.resource_api.create_domain(domain['id'], domain)
+            return PROVIDERS.resource_api.create_domain(domain['id'], domain)
 
         self.domains = {}
         self.addCleanup(self.cleanup_domains)
@@ -462,16 +465,16 @@ class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
         }
 
         # Clear backend dependencies, since cli loads these manually
-        dependency.reset()
+        provider_api.ProviderAPIs._clear_registry_instances()
         cli.DomainConfigUpload.main()
 
-        res = self.domain_config_api.get_config_with_sensitive_info(
+        res = PROVIDERS.domain_config_api.get_config_with_sensitive_info(
             CONF.identity.default_domain_id)
         self.assertEqual(default_config, res)
-        res = self.domain_config_api.get_config_with_sensitive_info(
+        res = PROVIDERS.domain_config_api.get_config_with_sensitive_info(
             self.domains['domain1']['id'])
         self.assertEqual(domain1_config, res)
-        res = self.domain_config_api.get_config_with_sensitive_info(
+        res = PROVIDERS.domain_config_api.get_config_with_sensitive_info(
             self.domains['domain2']['id'])
         self.assertEqual(domain2_config, res)
 
@@ -494,16 +497,16 @@ class CliDomainConfigSingleDomainTestCase(CliDomainConfigAllTestCase):
         }
 
         # Clear backend dependencies, since cli loads these manually
-        dependency.reset()
+        provider_api.ProviderAPIs._clear_registry_instances()
         cli.DomainConfigUpload.main()
 
-        res = self.domain_config_api.get_config_with_sensitive_info(
+        res = PROVIDERS.domain_config_api.get_config_with_sensitive_info(
             CONF.identity.default_domain_id)
         self.assertEqual(default_config, res)
-        res = self.domain_config_api.get_config_with_sensitive_info(
+        res = PROVIDERS.domain_config_api.get_config_with_sensitive_info(
             self.domains['domain1']['id'])
         self.assertEqual({}, res)
-        res = self.domain_config_api.get_config_with_sensitive_info(
+        res = PROVIDERS.domain_config_api.get_config_with_sensitive_info(
             self.domains['domain2']['id'])
         self.assertEqual({}, res)
 
@@ -513,12 +516,12 @@ class CliDomainConfigSingleDomainTestCase(CliDomainConfigAllTestCase):
             'ldap': {'url': uuid.uuid4().hex},
             'identity': {'driver': 'ldap'}
         }
-        self.domain_config_api.create_config(
+        PROVIDERS.domain_config_api.create_config(
             CONF.identity.default_domain_id, default_config)
 
         # Now try and upload the settings in the configuration file for the
         # default domain
-        dependency.reset()
+        provider_api.ProviderAPIs._clear_registry_instances()
         with mock.patch('six.moves.builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             file_name = ('keystone.%s.conf' % self.default_domain['name'])
@@ -530,7 +533,7 @@ class CliDomainConfigSingleDomainTestCase(CliDomainConfigAllTestCase):
                                          file_name)}
             mock_print.assert_has_calls([mock.call(error_msg)])
 
-        res = self.domain_config_api.get_config(
+        res = PROVIDERS.domain_config_api.get_config(
             CONF.identity.default_domain_id)
         # The initial config should not have been overwritten
         self.assertEqual(default_config, res)
@@ -543,7 +546,7 @@ class CliDomainConfigNoOptionsTestCase(CliDomainConfigAllTestCase):
              project='keystone', default_config_files=config_files)
 
     def test_config_upload(self):
-        dependency.reset()
+        provider_api.ProviderAPIs._clear_registry_instances()
         with mock.patch('six.moves.builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             mock_print.assert_has_calls(
@@ -560,7 +563,7 @@ class CliDomainConfigTooManyOptionsTestCase(CliDomainConfigAllTestCase):
              project='keystone', default_config_files=config_files)
 
     def test_config_upload(self):
-        dependency.reset()
+        provider_api.ProviderAPIs._clear_registry_instances()
         with mock.patch('six.moves.builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             mock_print.assert_has_calls(
@@ -577,7 +580,7 @@ class CliDomainConfigInvalidDomainTestCase(CliDomainConfigAllTestCase):
              project='keystone', default_config_files=config_files)
 
     def test_config_upload(self):
-        dependency.reset()
+        provider_api.ProviderAPIs._clear_registry_instances()
         with mock.patch('six.moves.builtins.print') as mock_print:
             self.assertRaises(unit.UnexpectedExit, cli.DomainConfigUpload.main)
             file_name = 'keystone.%s.conf' % self.invalid_domain_name
@@ -682,7 +685,7 @@ class CliDBSyncTestCase(unit.BaseTestCase):
 
     @mock.patch('keystone.cmd.cli.upgrades.get_db_version')
     def test_db_sync_check_when_database_is_empty(self, mocked_get_db_version):
-        e = migration.exception.DbMigrationError("Invalid version")
+        e = migration.exception.DBMigrationError("Invalid version")
         mocked_get_db_version.side_effect = e
         checker = cli.DbSync()
 
@@ -731,17 +734,20 @@ class TestMappingPopulate(unit.SQLDriverOverrides, unit.TestCase):
         # 3. Execute mapping_populate. It should create id mappings
         # 4. For the same users verify that they have public_id now
         purge_filter = {}
-        self.id_mapping_api.purge_mappings(purge_filter)
+        PROVIDERS.id_mapping_api.purge_mappings(purge_filter)
         hints = None
-        users = self.identity_api.driver.list_users(hints)
+        users = PROVIDERS.identity_api.driver.list_users(hints)
         for user in users:
             local_entity = {
                 'domain_id': CONF.identity.default_domain_id,
                 'local_id': user['id'],
                 'entity_type': identity_mapping.EntityType.USER}
-            self.assertIsNone(self.id_mapping_api.get_public_id(local_entity))
+            self.assertIsNone(
+                PROVIDERS.id_mapping_api.get_public_id(local_entity)
+            )
 
-        dependency.reset()  # backends are loaded again in the command handler
+        # backends are loaded again in the command handler
+        provider_api.ProviderAPIs._clear_registry_instances()
         cli.MappingPopulate.main()
 
         for user in users:
@@ -750,12 +756,13 @@ class TestMappingPopulate(unit.SQLDriverOverrides, unit.TestCase):
                 'local_id': user['id'],
                 'entity_type': identity_mapping.EntityType.USER}
             self.assertIsNotNone(
-                self.id_mapping_api.get_public_id(local_entity))
+                PROVIDERS.id_mapping_api.get_public_id(local_entity))
 
     def test_bad_domain_name(self):
         CONF(args=['mapping_populate', '--domain-name', uuid.uuid4().hex],
              project='keystone')
-        dependency.reset()  # backends are loaded again in the command handler
+        # backends are loaded again in the command handler
+        provider_api.ProviderAPIs._clear_registry_instances()
         # NOTE: assertEqual is used on purpose. assertFalse passes with None.
         self.assertEqual(False, cli.MappingPopulate.main())
 
@@ -851,7 +858,7 @@ class CredentialDoctorTests(unit.TestCase):
     def test_usability_of_cred_fernet_key_repo_raised(self, mock_utils):
         # Symptom Detected: credential fernet key repository is world readable
         self.config_fixture.config(group='credential', provider='fernet')
-        mock_utils.FernetUtils().validate_key_repository.return_value = False
+        mock_utils.TokenUtils().validate_key_repository.return_value = False
         self.assertTrue(
             credential.symptom_usability_of_credential_fernet_key_repository())
 
@@ -859,13 +866,13 @@ class CredentialDoctorTests(unit.TestCase):
     def test_usability_of_cred_fernet_key_repo_not_raised(self, mock_utils):
         # No Symptom Detected: Custom driver is used
         self.config_fixture.config(group='credential', provider='my-driver')
-        mock_utils.FernetUtils().validate_key_repository.return_value = True
+        mock_utils.TokenUtils().validate_key_repository.return_value = True
         self.assertFalse(
             credential.symptom_usability_of_credential_fernet_key_repository())
 
         # No Symptom Detected: key repository is not world readable
         self.config_fixture.config(group='credential', provider='fernet')
-        mock_utils.FernetUtils().validate_key_repository.return_value = True
+        mock_utils.TokenUtils().validate_key_repository.return_value = True
         self.assertFalse(
             credential.symptom_usability_of_credential_fernet_key_repository())
 
@@ -873,7 +880,7 @@ class CredentialDoctorTests(unit.TestCase):
     def test_keys_in_credential_fernet_key_repository_raised(self, mock_utils):
         # Symptom Detected: Key repo is empty
         self.config_fixture.config(group='credential', provider='fernet')
-        mock_utils.FernetUtils().load_keys.return_value = False
+        mock_utils.TokenUtils().load_keys.return_value = False
         self.assertTrue(
             credential.symptom_keys_in_credential_fernet_key_repository())
 
@@ -882,13 +889,13 @@ class CredentialDoctorTests(unit.TestCase):
             self, mock_utils):
         # No Symptom Detected: Custom driver is used
         self.config_fixture.config(group='credential', provider='my-driver')
-        mock_utils.FernetUtils().load_keys.return_value = True
+        mock_utils.TokenUtils().load_keys.return_value = True
         self.assertFalse(
             credential.symptom_keys_in_credential_fernet_key_repository())
 
         # No Symptom Detected: Key repo is not empty, fernet is current driver
         self.config_fixture.config(group='credential', provider='fernet')
-        mock_utils.FernetUtils().load_keys.return_value = True
+        mock_utils.TokenUtils().load_keys.return_value = True
         self.assertFalse(
             credential.symptom_keys_in_credential_fernet_key_repository())
 
@@ -1272,7 +1279,7 @@ class TokenFernetDoctorTests(unit.TestCase):
     def test_usability_of_Fernet_key_repository_raised(self, mock_utils):
         # Symptom Detected: Fernet key repo is world readable
         self.config_fixture.config(group='token', provider='fernet')
-        mock_utils.FernetUtils().validate_key_repository.return_value = False
+        mock_utils.TokenUtils().validate_key_repository.return_value = False
         self.assertTrue(
             tokens_fernet.symptom_usability_of_Fernet_key_repository())
 
@@ -1280,14 +1287,14 @@ class TokenFernetDoctorTests(unit.TestCase):
     def test_usability_of_Fernet_key_repository_not_raised(self, mock_utils):
         # No Symptom Detected: UUID is used instead of fernet
         self.config_fixture.config(group='token', provider='uuid')
-        mock_utils.FernetUtils().validate_key_repository.return_value = False
+        mock_utils.TokenUtils().validate_key_repository.return_value = False
         self.assertFalse(
             tokens_fernet.symptom_usability_of_Fernet_key_repository())
 
         # No Symptom Detected: configs set properly, key repo is not world
         # readable but is user readable
         self.config_fixture.config(group='token', provider='fernet')
-        mock_utils.FernetUtils().validate_key_repository.return_value = True
+        mock_utils.TokenUtils().validate_key_repository.return_value = True
         self.assertFalse(
             tokens_fernet.symptom_usability_of_Fernet_key_repository())
 
@@ -1295,7 +1302,7 @@ class TokenFernetDoctorTests(unit.TestCase):
     def test_keys_in_Fernet_key_repository_raised(self, mock_utils):
         # Symptom Detected: Fernet key repository is empty
         self.config_fixture.config(group='token', provider='fernet')
-        mock_utils.FernetUtils().load_keys.return_value = False
+        mock_utils.TokenUtils().load_keys.return_value = False
         self.assertTrue(
             tokens_fernet.symptom_keys_in_Fernet_key_repository())
 
@@ -1303,13 +1310,95 @@ class TokenFernetDoctorTests(unit.TestCase):
     def test_keys_in_Fernet_key_repository_not_raised(self, mock_utils):
         # No Symptom Detected: UUID is used instead of fernet
         self.config_fixture.config(group='token', provider='uuid')
-        mock_utils.FernetUtils().load_keys.return_value = True
+        mock_utils.TokenUtils().load_keys.return_value = True
         self.assertFalse(
             tokens_fernet.symptom_usability_of_Fernet_key_repository())
 
         # No Symptom Detected: configs set properly, key repo has been
         # populated with keys
         self.config_fixture.config(group='token', provider='fernet')
-        mock_utils.FernetUtils().load_keys.return_value = True
+        mock_utils.TokenUtils().load_keys.return_value = True
         self.assertFalse(
             tokens_fernet.symptom_usability_of_Fernet_key_repository())
+
+
+class TestMappingPurge(unit.SQLDriverOverrides, unit.BaseTestCase):
+
+    class FakeConfCommand(object):
+        def __init__(self, parent):
+            self.extension = False
+            self.all = parent.command_all
+            self.type = parent.command_type
+            self.domain_name = parent.command_domain_name
+            self.local_id = parent.command_local_id
+            self.public_id = parent.command_public_id
+
+    def setUp(self):
+        # Set up preset cli options and a parser
+        super(TestMappingPurge, self).setUp()
+        self.config_fixture = self.useFixture(oslo_config.fixture.Config(CONF))
+        self.config_fixture.register_cli_opt(cli.command_opt)
+        # For unit tests that should not throw any erorrs,
+        # Use the argument parser to test that the combinations work
+        parser_test = argparse.ArgumentParser()
+        subparsers = parser_test.add_subparsers()
+        self.parser = cli.MappingPurge.add_argument_parser(subparsers)
+
+    def test_mapping_purge_with_no_arguments_fails(self):
+        # Make sure the logic in main() actually catches no argument error
+        self.command_type = None
+        self.command_all = False
+        self.command_domain_name = None
+        self.command_local_id = None
+        self.command_public_id = None
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        self.assertRaises(ValueError, cli.MappingPurge.main)
+
+    def test_mapping_purge_with_all_and_other_argument_fails(self):
+        # Make sure the logic in main() actually catches invalid combinations
+        self.command_type = 'user'
+        self.command_all = True
+        self.command_domain_name = None
+        self.command_local_id = None
+        self.command_public_id = None
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        self.assertRaises(ValueError, cli.MappingPurge.main)
+
+    def test_mapping_purge_with_only_all_passes(self):
+        args = (['--all'])
+        res = self.parser.parse_args(args)
+        self.assertTrue(vars(res)['all'])
+
+    def test_mapping_purge_with_domain_name_argument_succeeds(self):
+        args = (['--domain-name', uuid.uuid4().hex])
+        self.parser.parse_args(args)
+
+    def test_mapping_purge_with_public_id_argument_succeeds(self):
+        args = (['--public-id', uuid.uuid4().hex])
+        self.parser.parse_args(args)
+
+    def test_mapping_purge_with_local_id_argument_succeeds(self):
+        args = (['--local-id', uuid.uuid4().hex])
+        self.parser.parse_args(args)
+
+    def test_mapping_purge_with_type_argument_succeeds(self):
+        args = (['--type', 'user'])
+        self.parser.parse_args(args)
+        args = (['--type', 'group'])
+        self.parser.parse_args(args)
+
+    def test_mapping_purge_with_invalid_argument_fails(self):
+        args = (['--invalid-option', 'some value'])
+        self.assertRaises(unit.UnexpectedExit, self.parser.parse_args, args)
+
+    def test_mapping_purge_with_all_other_combinations_passes(self):
+        args = (['--type', 'user', '--local-id', uuid.uuid4().hex])
+        self.parser.parse_args(args)
+        args.append('--domain-name')
+        args.append('test')
+        self.parser.parse_args(args)
+        args.append('--public-id')
+        args.append(uuid.uuid4().hex)
+        self.parser.parse_args(args)
