@@ -13,11 +13,11 @@
 # under the License.
 
 import subprocess
-import uuid
 
 import ldap.modlist
 from six.moves import range
 
+from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
 from keystone.identity.backends import ldap as identity_ldap
@@ -26,6 +26,7 @@ from keystone.tests.unit import test_backend_ldap
 
 
 CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 def create_object(dn, attrs):
@@ -66,9 +67,6 @@ class LiveLDAPIdentity(test_backend_ldap.LDAPIdentity):
         create_object(CONF.ldap.user_tree_dn,
                       {'objectclass': 'organizationalUnit',
                        'ou': 'Users'})
-        create_object(CONF.ldap.role_tree_dn,
-                      {'objectclass': 'organizationalUnit',
-                       'ou': 'Roles'})
         create_object(CONF.ldap.group_tree_dn,
                       {'objectclass': 'organizationalUnit',
                        'ou': 'UserGroups'})
@@ -104,20 +102,20 @@ class LiveLDAPIdentity(test_backend_ldap.LDAPIdentity):
         self.config_fixture.config(group='ldap',
                                    query_scope='sub',
                                    alias_dereferencing='never')
-        self.identity_api = identity_ldap.Identity()
+        PROVIDERS.identity_api = identity_ldap.Identity()
         self.assertRaises(exception.UserNotFound,
-                          self.identity_api.get_user,
+                          PROVIDERS.identity_api.get_user,
                           'alt_fake1')
 
         self.config_fixture.config(group='ldap',
                                    alias_dereferencing='searching')
-        self.identity_api = identity_ldap.Identity()
-        user_ref = self.identity_api.get_user('alt_fake1')
+        PROVIDERS.identity_api = identity_ldap.Identity()
+        user_ref = PROVIDERS.identity_api.get_user('alt_fake1')
         self.assertEqual('alt_fake1', user_ref['id'])
 
         self.config_fixture.config(group='ldap', alias_dereferencing='always')
-        self.identity_api = identity_ldap.Identity()
-        user_ref = self.identity_api.get_user('alt_fake1')
+        PROVIDERS.identity_api = identity_ldap.Identity()
+        user_ref = PROVIDERS.identity_api.get_user('alt_fake1')
         self.assertEqual('alt_fake1', user_ref['id'])
 
     # FakeLDAP does not correctly process filters, so this test can only be
@@ -129,57 +127,51 @@ class LiveLDAPIdentity(test_backend_ldap.LDAPIdentity):
         GROUP_COUNT = 3
         USER_COUNT = 2
 
-        for x in range(0, USER_COUNT):
-            # TODO(shaleh): use unit.new_user_ref()
-            new_user = {'name': uuid.uuid4().hex, 'password': uuid.uuid4().hex,
-                        'enabled': True, 'domain_id': domain['id']}
-            new_user = self.identity_api.create_user(new_user)
-            test_users.append(new_user)
-        positive_user = test_users[0]
-        negative_user = test_users[1]
+        positive_user = unit.create_user(PROVIDERS.identity_api, domain['id'])
+        negative_user = unit.create_user(PROVIDERS.identity_api, domain['id'])
 
         for x in range(0, USER_COUNT):
-            group_refs = self.identity_api.list_groups_for_user(
+            group_refs = PROVIDERS.identity_api.list_groups_for_user(
                 test_users[x]['id'])
             self.assertEqual(0, len(group_refs))
 
         for x in range(0, GROUP_COUNT):
             new_group = unit.new_group_ref(domain_id=domain['id'])
-            new_group = self.identity_api.create_group(new_group)
+            new_group = PROVIDERS.identity_api.create_group(new_group)
             test_groups.append(new_group)
 
-            group_refs = self.identity_api.list_groups_for_user(
+            group_refs = PROVIDERS.identity_api.list_groups_for_user(
                 positive_user['id'])
             self.assertEqual(x, len(group_refs))
 
-            self.identity_api.add_user_to_group(
+            PROVIDERS.identity_api.add_user_to_group(
                 positive_user['id'],
                 new_group['id'])
-            group_refs = self.identity_api.list_groups_for_user(
+            group_refs = PROVIDERS.identity_api.list_groups_for_user(
                 positive_user['id'])
             self.assertEqual(x + 1, len(group_refs))
 
-            group_refs = self.identity_api.list_groups_for_user(
+            group_refs = PROVIDERS.identity_api.list_groups_for_user(
                 negative_user['id'])
             self.assertEqual(0, len(group_refs))
 
-        driver = self.identity_api._select_identity_driver(
+        driver = PROVIDERS.identity_api._select_identity_driver(
             CONF.identity.default_domain_id)
         driver.group.ldap_filter = '(dn=xx)'
 
-        group_refs = self.identity_api.list_groups_for_user(
+        group_refs = PROVIDERS.identity_api.list_groups_for_user(
             positive_user['id'])
         self.assertEqual(0, len(group_refs))
-        group_refs = self.identity_api.list_groups_for_user(
+        group_refs = PROVIDERS.identity_api.list_groups_for_user(
             negative_user['id'])
         self.assertEqual(0, len(group_refs))
 
         driver.group.ldap_filter = '(objectclass=*)'
 
-        group_refs = self.identity_api.list_groups_for_user(
+        group_refs = PROVIDERS.identity_api.list_groups_for_user(
             positive_user['id'])
         self.assertEqual(GROUP_COUNT, len(group_refs))
-        group_refs = self.identity_api.list_groups_for_user(
+        group_refs = PROVIDERS.identity_api.list_groups_for_user(
             negative_user['id'])
         self.assertEqual(0, len(group_refs))
 
