@@ -12,20 +12,20 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import uuid
-
 import ldappool
-from oslo_config import cfg
 
-from keystone.common.ldap import core as ldap_core
+from keystone.common import provider_api
+import keystone.conf
 from keystone.identity.backends import ldap
+from keystone.identity.backends.ldap import common as ldap_common
 from keystone.tests import unit
 from keystone.tests.unit import fakeldap
 from keystone.tests.unit import test_backend_ldap_pool
 from keystone.tests.unit import test_ldap_livetest
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
@@ -40,7 +40,7 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
         super(LiveLDAPPoolIdentity, self).setUp()
         self.addCleanup(self.cleanup_pools)
         # storing to local variable to avoid long references
-        self.conn_pools = ldap_core.PooledLDAPHandler.connection_pools
+        self.conn_pools = ldap_common.PooledLDAPHandler.connection_pools
 
     def config_files(self):
         config_files = super(LiveLDAPPoolIdentity, self).config_files()
@@ -48,7 +48,7 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
         return config_files
 
     def test_assert_connector_used_not_fake_ldap_pool(self):
-        handler = ldap_core._get_connection(CONF.ldap.url, use_pool=True)
+        handler = ldap_common._get_connection(CONF.ldap.url, use_pool=True)
         self.assertNotEqual(type(handler.Connector),
                             type(fakeldap.FakeLdapPool))
         self.assertEqual(type(ldappool.StateConnector),
@@ -85,7 +85,7 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
                 self.assertEqual(2, len(ldappool_cm))
                 self.assertTrue(c3.connected)
                 self.assertTrue(c3.active)
-                self.assertTrue(c3 is c2)  # same connection is reused
+                self.assertIs(c3, c2)  # same connection is reused
                 self.assertTrue(c2.active)
                 with _get_conn() as c4:  # conn4
                     self.assertEqual(3, len(ldappool_cm))
@@ -99,28 +99,26 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
         self.test_password_change_with_pool()
 
         self.assertRaises(AssertionError,
-                          self.identity_api.authenticate,
+                          PROVIDERS.identity_api.authenticate,
                           context={},
                           user_id=self.user_sna['id'],
                           password=old_password)
 
     def _create_user_and_authenticate(self, password):
-        # TODO(shaleh): port to new_user_ref()
-        user_dict = {
-            'domain_id': CONF.identity.default_domain_id,
-            'name': uuid.uuid4().hex,
-            'password': password}
-        user = self.identity_api.create_user(user_dict)
+        user = unit.create_user(PROVIDERS.identity_api,
+                                CONF.identity.default_domain_id,
+                                password=password)
 
-        self.identity_api.authenticate(
-            context={},
+        PROVIDERS.identity_api.authenticate(
+            self.make_request(),
             user_id=user['id'],
             password=password)
 
-        return self.identity_api.get_user(user['id'])
+        return PROVIDERS.identity_api.get_user(user['id'])
 
     def _get_auth_conn_pool_cm(self):
-        pool_url = ldap_core.PooledLDAPHandler.auth_pool_prefix + CONF.ldap.url
+        pool_url = (
+            ldap_common.PooledLDAPHandler.auth_pool_prefix + CONF.ldap.url)
         return self.conn_pools[pool_url]
 
     def _do_password_change_for_one_user(self, password, new_password):
@@ -164,7 +162,7 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
                             _.unbind_s()
 
         user3['password'] = new_password
-        self.identity_api.update_user(user3['id'], user3)
+        PROVIDERS.identity_api.update_user(user3['id'], user3)
 
         return user3
 
@@ -181,8 +179,8 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
         # successfully which is not desired if password change is frequent
         # use case in a deployment.
         # This can happen in multiple concurrent connections case only.
-        user_ref = self.identity_api.authenticate(
-            context={}, user_id=user['id'], password=old_password)
+        user_ref = PROVIDERS.identity_api.authenticate(
+            self.make_request(), user_id=user['id'], password=old_password)
 
         self.assertDictEqual(user, user_ref)
 
@@ -197,6 +195,6 @@ class LiveLDAPPoolIdentity(test_backend_ldap_pool.LdapPoolCommonTestMixin,
         # now as connection lifetime is zero, so authentication
         # with old password will always fail.
         self.assertRaises(AssertionError,
-                          self.identity_api.authenticate,
+                          PROVIDERS.identity_api.authenticate,
                           context={}, user_id=user['id'],
                           password=old_password)
