@@ -23,7 +23,7 @@ from oslo_log import log
 from six.moves import http_client
 from testtools import matchers
 
-from keystone.common import controller
+from keystone.common import provider_api
 from keystone.common import sql
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
@@ -39,25 +39,7 @@ from keystone.tests.unit import test_v3
 
 
 CONF = keystone.conf.CONF
-
-
-# NOTE(morganfainberg): To be removed when admin_token_auth middleware is
-# removed. This was moved to it's own testcase so it can setup the
-# admin_token_auth pipeline without impacting other tests.
-class IdentityTestCaseStaticAdminToken(test_v3.RestfulTestCase):
-
-    def config_overrides(self):
-        super(IdentityTestCaseStaticAdminToken, self).config_overrides()
-        self.config_fixture.config(
-            admin_token='ADMIN')
-
-    def test_list_users_with_static_admin_token_and_multiple_backends(self):
-        # domain-specific operations with the bootstrap ADMIN token is
-        # disallowed when domain-specific drivers are enabled
-        self.config_fixture.config(group='identity',
-                                   domain_specific_drivers_enabled=True)
-        self.get('/users', token=CONF.admin_token,
-                 expected_status=exception.Unauthorized.code)
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class IdentityTestCase(test_v3.RestfulTestCase):
@@ -74,15 +56,16 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         )
 
         self.group = unit.new_group_ref(domain_id=self.domain_id)
-        self.group = self.identity_api.create_group(self.group)
+        self.group = PROVIDERS.identity_api.create_group(self.group)
         self.group_id = self.group['id']
 
         self.credential = unit.new_credential_ref(
             user_id=self.user['id'],
             project_id=self.project_id)
 
-        self.credential_api.create_credential(self.credential['id'],
-                                              self.credential)
+        PROVIDERS.credential_api.create_credential(
+            self.credential['id'], self.credential
+        )
 
     # user crud tests
 
@@ -105,9 +88,9 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         # Create a user with a role on the domain so we can get a
         # domain scoped token
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
-        user = unit.create_user(self.identity_api, domain_id=domain['id'])
-        self.assignment_api.create_grant(
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
+        user = unit.create_user(PROVIDERS.identity_api, domain_id=domain['id'])
+        PROVIDERS.assignment_api.create_grant(
             role_id=self.role_id, user_id=user['id'],
             domain_id=domain['id'])
 
@@ -171,9 +154,9 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         """
         # Create two domains to work with.
         domain1 = unit.new_domain_ref()
-        self.resource_api.create_domain(domain1['id'], domain1)
+        PROVIDERS.resource_api.create_domain(domain1['id'], domain1)
         domain2 = unit.new_domain_ref()
-        self.resource_api.create_domain(domain2['id'], domain2)
+        PROVIDERS.resource_api.create_domain(domain2['id'], domain2)
 
         # We can successfully create a normal user without any surprises.
         user = unit.new_user_ref(domain_id=domain1['id'])
@@ -250,19 +233,19 @@ class IdentityTestCase(test_v3.RestfulTestCase):
 
         # Create a new domain with a new project and user
         domain = unit.new_domain_ref()
-        self.resource_api.create_domain(domain['id'], domain)
+        PROVIDERS.resource_api.create_domain(domain['id'], domain)
 
         project = unit.new_project_ref(domain_id=domain['id'])
-        self.resource_api.create_project(project['id'], project)
+        PROVIDERS.resource_api.create_project(project['id'], project)
 
-        user = unit.create_user(self.identity_api, domain_id=domain['id'])
+        user = unit.create_user(PROVIDERS.identity_api, domain_id=domain['id'])
 
         # Create both project and domain role grants for the user so we
         # can get both project and domain scoped tokens
-        self.assignment_api.create_grant(
+        PROVIDERS.assignment_api.create_grant(
             role_id=self.role_id, user_id=user['id'],
             domain_id=domain['id'])
-        self.assignment_api.create_grant(
+        PROVIDERS.assignment_api.create_grant(
             role_id=self.role_id, user_id=user['id'],
             project_id=project['id'])
 
@@ -297,7 +280,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
     def test_list_users_no_default_project(self):
         """Call ``GET /users`` making sure no default_project_id."""
         user = unit.new_user_ref(self.domain_id)
-        user = self.identity_api.create_user(user)
+        user = PROVIDERS.identity_api.create_user(user)
         resource_url = '/users'
         r = self.get(resource_url)
         self.assertValidUserListResponse(r, ref=user,
@@ -315,7 +298,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         """Call ``GET /users/{user_id}`` extra attributes are not included."""
         user = unit.new_user_ref(domain_id=self.domain_id,
                                  project_id=self.project_id)
-        user = self.identity_api.create_user(user)
+        user = PROVIDERS.identity_api.create_user(user)
         self.assertNotIn('created_at', user)
         self.assertNotIn('last_active_at', user)
 
@@ -323,7 +306,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         """Call ``GET /users/{user_id}`` required attributes are included."""
         user = unit.new_user_ref(domain_id=self.domain_id,
                                  project_id=self.project_id)
-        user = self.identity_api.create_user(user)
+        user = PROVIDERS.identity_api.create_user(user)
         self.assertIn('id', user)
         self.assertIn('name', user)
         self.assertIn('enabled', user)
@@ -335,7 +318,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         """Call ``GET /users/{user_id}`` making sure of default_project_id."""
         user = unit.new_user_ref(domain_id=self.domain_id,
                                  project_id=self.project_id)
-        user = self.identity_api.create_user(user)
+        user = PROVIDERS.identity_api.create_user(user)
         r = self.get('/users/%(user_id)s' % {'user_id': user['id']})
         self.assertValidUserResponse(r, user)
 
@@ -346,9 +329,9 @@ class IdentityTestCase(test_v3.RestfulTestCase):
 
     def test_list_head_groups_for_user(self):
         """Call ``GET & HEAD /users/{user_id}/groups``."""
-        user1 = unit.create_user(self.identity_api,
+        user1 = unit.create_user(PROVIDERS.identity_api,
                                  domain_id=self.domain['id'])
-        user2 = unit.create_user(self.identity_api,
+        user2 = unit.create_user(PROVIDERS.identity_api,
                                  domain_id=self.domain['id'])
 
         self.put('/groups/%(group_id)s/users/%(user_id)s' % {
@@ -424,7 +407,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
 
     def test_admin_password_reset(self):
         # bootstrap a user as admin
-        user_ref = unit.create_user(self.identity_api,
+        user_ref = unit.create_user(PROVIDERS.identity_api,
                                     domain_id=self.domain['id'])
 
         # auth as user should work before a password change
@@ -463,7 +446,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         self.config_fixture.config(group='security_compliance',
                                    minimum_password_age=1)
         # create user
-        user_ref = unit.create_user(self.identity_api,
+        user_ref = unit.create_user(PROVIDERS.identity_api,
                                     domain_id=self.domain['id'])
         # administrative password reset
         new_password = uuid.uuid4().hex
@@ -483,7 +466,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         the `domain_id` of a user fails.
         """
         user = unit.new_user_ref(domain_id=self.domain['id'])
-        user = self.identity_api.create_user(user)
+        user = PROVIDERS.identity_api.create_user(user)
         user['domain_id'] = CONF.identity.default_domain_id
         self.patch('/users/%(user_id)s' % {
             'user_id': user['id']},
@@ -500,16 +483,18 @@ class IdentityTestCase(test_v3.RestfulTestCase):
 
         """
         # First check the credential for this user is present
-        r = self.credential_api.get_credential(self.credential['id'])
+        r = PROVIDERS.credential_api.get_credential(self.credential['id'])
         self.assertDictEqual(self.credential, r)
         # Create a second credential with a different user
 
         user2 = unit.new_user_ref(domain_id=self.domain['id'],
                                   project_id=self.project['id'])
-        user2 = self.identity_api.create_user(user2)
+        user2 = PROVIDERS.identity_api.create_user(user2)
         credential2 = unit.new_credential_ref(user_id=user2['id'],
                                               project_id=self.project['id'])
-        self.credential_api.create_credential(credential2['id'], credential2)
+        PROVIDERS.credential_api.create_credential(
+            credential2['id'], credential2
+        )
 
         # Create a token for this user which we can check later
         # gets deleted
@@ -530,14 +515,14 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         # Deleting the user should have deleted any credentials
         # that reference this project
         self.assertRaises(exception.CredentialNotFound,
-                          self.credential_api.get_credential,
+                          PROVIDERS.credential_api.get_credential,
                           self.credential['id'])
         # And the no tokens we remain valid
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             self.user['id'])
         self.assertEqual(0, len(tokens))
         # But the credential for user2 is unaffected
-        r = self.credential_api.get_credential(credential2['id'])
+        r = PROVIDERS.credential_api.get_credential(credential2['id'])
         self.assertDictEqual(credential2, r)
 
     def test_delete_user_retries_on_deadlock(self):
@@ -562,11 +547,11 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         side_effect = FakeDeadlock(patcher)
         sql_delete_mock.side_effect = side_effect
 
-        user_ref = unit.create_user(self.identity_api,
+        user_ref = unit.create_user(PROVIDERS.identity_api,
                                     domain_id=self.domain['id'])
 
         try:
-            self.identity_api.delete_user(user_id=user_ref['id'])
+            PROVIDERS.identity_api.delete_user(user_id=user_ref['id'])
         finally:
             if side_effect.patched:
                 patcher.stop()
@@ -654,7 +639,7 @@ class IdentityTestCase(test_v3.RestfulTestCase):
         log_fix = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
 
         # bootstrap a user as admin
-        user_ref = unit.create_user(self.identity_api,
+        user_ref = unit.create_user(PROVIDERS.identity_api,
                                     domain_id=self.domain['id'])
 
         self.assertNotIn(user_ref['password'], log_fix.output)
@@ -693,114 +678,11 @@ class IdentityTestCase(test_v3.RestfulTestCase):
             expected_status=http_client.BAD_REQUEST)
 
 
-class IdentityV3toV2MethodsTestCase(unit.TestCase):
-    """Test users V3 to V2 conversion methods."""
-
-    def new_user_ref(self, **kwargs):
-        """Construct a bare bones user ref.
-
-        Omits all optional components.
-        """
-        ref = unit.new_user_ref(**kwargs)
-        # description is already omitted
-        del ref['email']
-        del ref['enabled']
-        del ref['password']
-        return ref
-
-    def setUp(self):
-        super(IdentityV3toV2MethodsTestCase, self).setUp()
-        self.load_backends()
-        user_id = uuid.uuid4().hex
-        project_id = uuid.uuid4().hex
-
-        # User with only default_project_id in ref
-        self.user1 = self.new_user_ref(
-            id=user_id,
-            name=user_id,
-            project_id=project_id,
-            domain_id=CONF.identity.default_domain_id)
-        # User without default_project_id or tenantId in ref
-        self.user2 = self.new_user_ref(
-            id=user_id,
-            name=user_id,
-            domain_id=CONF.identity.default_domain_id)
-        # User with both tenantId and default_project_id in ref
-        self.user3 = self.new_user_ref(
-            id=user_id,
-            name=user_id,
-            project_id=project_id,
-            tenantId=project_id,
-            domain_id=CONF.identity.default_domain_id)
-        # User with only tenantId in ref
-        self.user4 = self.new_user_ref(
-            id=user_id,
-            name=user_id,
-            tenantId=project_id,
-            domain_id=CONF.identity.default_domain_id)
-        # User with password_expires_at
-        self.user5 = self.new_user_ref(
-            id=user_id,
-            name=user_id,
-            project_id=project_id,
-            domain_id=CONF.identity.default_domain_id,
-            password_expires_at=None)
-
-        # Expected result if the user is meant to have a tenantId element
-        self.expected_user = {'id': user_id,
-                              'name': user_id,
-                              'username': user_id,
-                              'tenantId': project_id}
-
-        # Expected result if the user is not meant to have a tenantId element
-        self.expected_user_no_tenant_id = {'id': user_id,
-                                           'name': user_id,
-                                           'username': user_id}
-
-    def test_v3_to_v2_user_method(self):
-
-        updated_user1 = controller.V2Controller.v3_to_v2_user(self.user1)
-        self.assertIs(self.user1, updated_user1)
-        self.assertDictEqual(self.expected_user, self.user1)
-        updated_user2 = controller.V2Controller.v3_to_v2_user(self.user2)
-        self.assertIs(self.user2, updated_user2)
-        self.assertDictEqual(self.expected_user_no_tenant_id, self.user2)
-        updated_user3 = controller.V2Controller.v3_to_v2_user(self.user3)
-        self.assertIs(self.user3, updated_user3)
-        self.assertDictEqual(self.expected_user, self.user3)
-        updated_user4 = controller.V2Controller.v3_to_v2_user(self.user4)
-        self.assertIs(self.user4, updated_user4)
-        self.assertDictEqual(self.expected_user_no_tenant_id, self.user4)
-        # password_expires_at filter test
-        password_expires_at_key = 'password_expires_at'
-        self.assertIn(password_expires_at_key, self.user5)
-        updated_user5 = controller.V2Controller.v3_to_v2_user(self.user5)
-        self.assertIs(self.user5, updated_user5)
-        self.assertNotIn(password_expires_at_key, updated_user5)
-
-    def test_v3_to_v2_user_method_list(self):
-        user_list = [self.user1, self.user2, self.user3, self.user4,
-                     self.user5]
-        updated_list = controller.V2Controller.v3_to_v2_user(user_list)
-
-        self.assertEqual(len(user_list), len(updated_list))
-
-        for i, ref in enumerate(updated_list):
-            # Order should not change.
-            self.assertIs(ref, user_list[i])
-            self.assertNotIn('password_expires_at', user_list[i])
-
-        self.assertDictEqual(self.expected_user, self.user1)
-        self.assertDictEqual(self.expected_user_no_tenant_id, self.user2)
-        self.assertDictEqual(self.expected_user, self.user3)
-        self.assertDictEqual(self.expected_user_no_tenant_id, self.user4)
-
-
 class ChangePasswordTestCase(test_v3.RestfulTestCase):
 
     def setUp(self):
         super(ChangePasswordTestCase, self).setUp()
-        self.user_ref = unit.create_user(self.identity_api,
+        self.user_ref = unit.create_user(PROVIDERS.identity_api,
                                          domain_id=self.domain['id'])
         self.token = self.get_request_token(self.user_ref['password'],
                                             http_client.CREATED)
@@ -833,7 +715,7 @@ class UserSelfServiceChangingPasswordsTestCase(ChangePasswordTestCase):
         user_ref = unit.new_user_ref(domain_id=self.domain_id,
                                      password=password)
         with freezegun.freeze_time(time):
-            self.user_ref = self.identity_api.create_user(user_ref)
+            self.user_ref = PROVIDERS.identity_api.create_user(user_ref)
 
         return password
 
@@ -966,7 +848,7 @@ class UserSelfServiceChangingPasswordsTestCase(ChangePasswordTestCase):
                                    change_password_upon_first_use=True)
 
         # create user
-        self.user_ref = unit.create_user(self.identity_api,
+        self.user_ref = unit.create_user(PROVIDERS.identity_api,
                                          domain_id=self.domain['id'])
 
         # attempt to authenticate with create user password
@@ -989,7 +871,7 @@ class UserSelfServiceChangingPasswordsTestCase(ChangePasswordTestCase):
         # admin reset
         reset_password = uuid.uuid4().hex
         user_password = {'password': reset_password}
-        self.identity_api.update_user(self.user_ref['id'], user_password)
+        PROVIDERS.identity_api.update_user(self.user_ref['id'], user_password)
 
         # attempt to authenticate with admin reset password
         self.get_request_token(reset_password,
@@ -1013,7 +895,7 @@ class UserSelfServiceChangingPasswordsTestCase(ChangePasswordTestCase):
         self.user_ref['password'] = reset_password
         ignore_opt_name = options.IGNORE_CHANGE_PASSWORD_OPT.option_name
         self.user_ref['options'][ignore_opt_name] = True
-        self.identity_api.update_user(self.user_ref['id'], self.user_ref)
+        PROVIDERS.identity_api.update_user(self.user_ref['id'], self.user_ref)
 
         # authenticate with the reset password
         self.token = self.get_request_token(reset_password,
@@ -1024,13 +906,13 @@ class UserSelfServiceChangingPasswordsTestCase(ChangePasswordTestCase):
                                    lockout_failure_attempts=1)
 
         # create user
-        self.user_ref = unit.create_user(self.identity_api,
+        self.user_ref = unit.create_user(PROVIDERS.identity_api,
                                          domain_id=self.domain['id'])
 
         # update the user, mark her as exempt from lockout
         ignore_opt_name = options.IGNORE_LOCKOUT_ATTEMPT_OPT.option_name
         self.user_ref['options'][ignore_opt_name] = True
-        self.identity_api.update_user(self.user_ref['id'], self.user_ref)
+        PROVIDERS.identity_api.update_user(self.user_ref['id'], self.user_ref)
 
         # fail to auth, this should lockout the user, since we're allowed
         # one failure, but we're exempt from lockout!
@@ -1059,7 +941,7 @@ class PasswordValidationTestCase(ChangePasswordTestCase):
                   expected_status=http_client.BAD_REQUEST)
 
     def test_update_user_with_invalid_password(self):
-        user = unit.create_user(self.identity_api,
+        user = unit.create_user(PROVIDERS.identity_api,
                                 domain_id=self.domain['id'])
         user['password'] = 'simple'
         self.patch('/users/%(user_id)s' % {
@@ -1105,19 +987,19 @@ class UserFederatedAttributesTests(test_v3.RestfulTestCase):
             'enabled': True,
             'description': uuid.uuid4().hex
         }
-        self.federation_api.create_idp(idp['id'], idp)
+        PROVIDERS.federation_api.create_idp(idp['id'], idp)
         # Create the mapping
         mapping = mapping_fixtures.MAPPING_EPHEMERAL_USER
         mapping['id'] = uuid.uuid4().hex
-        self.federation_api.create_mapping(mapping['id'], mapping)
+        PROVIDERS.federation_api.create_mapping(mapping['id'], mapping)
         # Create the protocol
         protocol = {
             'id': uuid.uuid4().hex,
             'mapping_id': mapping['id']
         }
-        self.federation_api.create_protocol(idp['id'],
-                                            protocol['id'],
-                                            protocol)
+        PROVIDERS.federation_api.create_protocol(
+            idp['id'], protocol['id'], protocol
+        )
         return idp, protocol
 
     def _create_user_with_federated_user(self, user, fed_dict):
@@ -1141,7 +1023,7 @@ class UserFederatedAttributesTests(test_v3.RestfulTestCase):
         self.fed_dict['unique_id'] = "jdoe"
         # Create the domain_id, user, and federated_user relationship
         self.domain = unit.new_domain_ref()
-        self.resource_api.create_domain(self.domain['id'], self.domain)
+        PROVIDERS.resource_api.create_domain(self.domain['id'], self.domain)
         self.fed_user = unit.new_user_ref(domain_id=self.domain['id'])
         self.fed_user = self._create_user_with_federated_user(self.fed_user,
                                                               self.fed_dict)
