@@ -19,12 +19,14 @@ from oslo_utils import timeutils
 import six
 from six.moves import range
 
+from keystone.common import provider_api
 from keystone import exception
 from keystone.tests import unit
 from keystone.token import provider
 
 
 NULL_OBJECT = object()
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class TokenTests(object):
@@ -37,7 +39,7 @@ class TokenTests(object):
         # token persistence service
         persistence_list = [
             x['id']
-            for x in self.token_provider_api.list_revoked_tokens()
+            for x in PROVIDERS.token_provider_api.list_revoked_tokens()
         ]
         self.assertEqual(persistence_list, revoked_token_id_list)
 
@@ -48,8 +50,9 @@ class TokenTests(object):
                 'user': {'id': 'testuserid'},
                 'token_data': {'access': {'token': {
                     'audit_ids': [uuid.uuid4().hex]}}}}
-        data_ref = self.token_provider_api._persistence.create_token(token_id,
-                                                                     data)
+        data_ref = PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, data
+        )
         expires = data_ref.pop('expires')
         data_ref.pop('user_id')
         self.assertIsInstance(expires, datetime.datetime)
@@ -57,7 +60,9 @@ class TokenTests(object):
         data.pop('id')
         self.assertDictEqual(data, data_ref)
 
-        new_data_ref = self.token_provider_api._persistence.get_token(token_id)
+        new_data_ref = PROVIDERS.token_provider_api._persistence.get_token(
+            token_id
+        )
         expires = new_data_ref.pop('expires')
         self.assertIsInstance(expires, datetime.datetime)
         new_data_ref.pop('user_id')
@@ -65,13 +70,13 @@ class TokenTests(object):
 
         self.assertEqual(data, new_data_ref)
 
-        self.token_provider_api._persistence.delete_token(token_id)
+        PROVIDERS.token_provider_api._persistence.delete_token(token_id)
         self.assertRaises(
             exception.TokenNotFound,
-            self.token_provider_api._persistence.get_token, token_id)
+            PROVIDERS.token_provider_api._persistence.get_token, token_id)
         self.assertRaises(
             exception.TokenNotFound,
-            self.token_provider_api._persistence.delete_token, token_id)
+            PROVIDERS.token_provider_api._persistence.delete_token, token_id)
 
     def create_token_sample_data(self, token_id=None, tenant_id=None,
                                  trust_id=None, user_id=None, expires=None):
@@ -97,16 +102,17 @@ class TokenTests(object):
             # the cases of impersonation and therefore should not match the
             # token's user_id.
             data['access']['trust']['trustee_user_id'] = 'testuserid2'
-        data['token_version'] = provider.V2
+        data['token_version'] = provider.V3
         # Issue token stores a copy of all token data at token['token_data'].
         # This emulates that assumption as part of the test.
         data['token_data'] = copy.deepcopy(data)
-        new_token = self.token_provider_api._persistence.create_token(token_id,
-                                                                      data)
+        new_token = PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, data
+        )
         return new_token['id'], data
 
     def test_delete_tokens(self):
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             'testuserid')
         self.assertEqual(0, len(tokens))
         token_id1, data = self.create_token_sample_data(
@@ -116,28 +122,28 @@ class TokenTests(object):
         token_id3, data = self.create_token_sample_data(
             tenant_id='testtenantid',
             user_id='testuserid1')
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             'testuserid')
         self.assertEqual(2, len(tokens))
         self.assertIn(token_id2, tokens)
         self.assertIn(token_id1, tokens)
-        self.token_provider_api._persistence.delete_tokens(
+        PROVIDERS.token_provider_api._persistence.delete_tokens(
             user_id='testuserid',
             tenant_id='testtenantid')
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             'testuserid')
         self.assertEqual(0, len(tokens))
         self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api._persistence.get_token,
+                          PROVIDERS.token_provider_api._persistence.get_token,
                           token_id1)
         self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api._persistence.get_token,
+                          PROVIDERS.token_provider_api._persistence.get_token,
                           token_id2)
 
-        self.token_provider_api._persistence.get_token(token_id3)
+        PROVIDERS.token_provider_api._persistence.get_token(token_id3)
 
     def test_delete_tokens_trust(self):
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             user_id='testuserid')
         self.assertEqual(0, len(tokens))
         token_id1, data = self.create_token_sample_data(
@@ -147,18 +153,18 @@ class TokenTests(object):
             tenant_id='testtenantid',
             user_id='testuserid1',
             trust_id='testtrustid1')
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             'testuserid')
         self.assertEqual(1, len(tokens))
         self.assertIn(token_id1, tokens)
-        self.token_provider_api._persistence.delete_tokens(
+        PROVIDERS.token_provider_api._persistence.delete_tokens(
             user_id='testuserid',
             tenant_id='testtenantid',
             trust_id='testtrustid')
         self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api._persistence.get_token,
+                          PROVIDERS.token_provider_api._persistence.get_token,
                           token_id1)
-        self.token_provider_api._persistence.get_token(token_id2)
+        PROVIDERS.token_provider_api._persistence.get_token(token_id2)
 
     def _test_token_list(self, token_list_fn):
         tokens = token_list_fn('testuserid')
@@ -172,11 +178,11 @@ class TokenTests(object):
         self.assertEqual(2, len(tokens))
         self.assertIn(token_id2, tokens)
         self.assertIn(token_id1, tokens)
-        self.token_provider_api._persistence.delete_token(token_id1)
+        PROVIDERS.token_provider_api._persistence.delete_token(token_id1)
         tokens = token_list_fn('testuserid')
         self.assertIn(token_id2, tokens)
         self.assertNotIn(token_id1, tokens)
-        self.token_provider_api._persistence.delete_token(token_id2)
+        PROVIDERS.token_provider_api._persistence.delete_token(token_id2)
         tokens = token_list_fn('testuserid')
         self.assertNotIn(token_id2, tokens)
         self.assertNotIn(token_id1, tokens)
@@ -204,34 +210,39 @@ class TokenTests(object):
 
     def test_token_list(self):
         self._test_token_list(
-            self.token_provider_api._persistence._list_tokens)
+            PROVIDERS.token_provider_api._persistence._list_tokens)
 
     def test_token_list_trust(self):
         trust_id = uuid.uuid4().hex
         token_id5, data = self.create_token_sample_data(trust_id=trust_id)
-        tokens = self.token_provider_api._persistence._list_tokens(
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             'testuserid', trust_id=trust_id)
         self.assertEqual(1, len(tokens))
         self.assertIn(token_id5, tokens)
 
     def test_get_token_returns_not_found(self):
         self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api._persistence.get_token,
+                          PROVIDERS.token_provider_api._persistence.get_token,
                           uuid.uuid4().hex)
 
     def test_delete_token_returns_not_found(self):
-        self.assertRaises(exception.TokenNotFound,
-                          self.token_provider_api._persistence.delete_token,
-                          uuid.uuid4().hex)
+        self.assertRaises(
+            exception.TokenNotFound,
+            PROVIDERS.token_provider_api._persistence.delete_token,
+            uuid.uuid4().hex
+        )
 
     def test_null_expires_token(self):
         token_id = uuid.uuid4().hex
         data = {'id': token_id, 'id_hash': token_id, 'a': 'b', 'expires': None,
                 'user': {'id': 'testuserid'}}
-        data_ref = self.token_provider_api._persistence.create_token(token_id,
-                                                                     data)
+        data_ref = PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, data
+        )
         self.assertIsNotNone(data_ref['expires'])
-        new_data_ref = self.token_provider_api._persistence.get_token(token_id)
+        new_data_ref = PROVIDERS.token_provider_api._persistence.get_token(
+            token_id
+        )
 
         # MySQL doesn't store microseconds, so discard them before testing
         data_ref['expires'] = data_ref['expires'].replace(microsecond=0)
@@ -241,7 +252,7 @@ class TokenTests(object):
         self.assertEqual(data_ref, new_data_ref)
 
     def check_list_revoked_tokens(self, token_infos):
-        revocation_list = self.token_provider_api.list_revoked_tokens()
+        revocation_list = PROVIDERS.token_provider_api.list_revoked_tokens()
         revoked_ids = [x['id'] for x in revocation_list]
         revoked_audit_ids = [x['audit_id'] for x in revocation_list]
         self._assert_revoked_token_list_matches_token_persistence(revoked_ids)
@@ -255,22 +266,24 @@ class TokenTests(object):
         data = {'id_hash': token_id, 'id': token_id, 'a': 'b',
                 'user': {'id': 'testuserid'},
                 'token_data': {'token': {'audit_ids': [audit_id]}}}
-        data_ref = self.token_provider_api._persistence.create_token(token_id,
-                                                                     data)
-        self.token_provider_api._persistence.delete_token(token_id)
+        data_ref = PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, data
+        )
+        PROVIDERS.token_provider_api._persistence.delete_token(token_id)
         self.assertRaises(
             exception.TokenNotFound,
-            self.token_provider_api._persistence.get_token,
+            PROVIDERS.token_provider_api._persistence.get_token,
             data_ref['id'])
         self.assertRaises(
             exception.TokenNotFound,
-            self.token_provider_api._persistence.delete_token,
+            PROVIDERS.token_provider_api._persistence.delete_token,
             data_ref['id'])
         return (token_id, audit_id)
 
     def test_list_revoked_tokens_returns_empty_list(self):
-        revoked_ids = [x['id']
-                       for x in self.token_provider_api.list_revoked_tokens()]
+        revoked_ids = [
+            x['id'] for x in PROVIDERS.token_provider_api.list_revoked_tokens()
+        ]
         self._assert_revoked_token_list_matches_token_persistence(revoked_ids)
         self.assertEqual([], revoked_ids)
 
@@ -289,8 +302,9 @@ class TokenTests(object):
                 'expires': expire_time,
                 'trust_id': None,
                 'user': {'id': 'testuserid'}}
-        data_ref = self.token_provider_api._persistence.create_token(token_id,
-                                                                     data)
+        data_ref = PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, data
+        )
         data_ref.pop('user_id')
         self.assertDictEqual(data, data_ref)
 
@@ -300,13 +314,14 @@ class TokenTests(object):
                 'expires': expire_time,
                 'trust_id': None,
                 'user': {'id': 'testuserid'}}
-        data_ref = self.token_provider_api._persistence.create_token(token_id,
-                                                                     data)
+        data_ref = PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, data
+        )
         data_ref.pop('user_id')
         self.assertDictEqual(data, data_ref)
 
-        self.token_provider_api._persistence.flush_expired_tokens()
-        tokens = self.token_provider_api._persistence._list_tokens(
+        PROVIDERS.token_provider_api._persistence.flush_expired_tokens()
+        tokens = PROVIDERS.token_provider_api._persistence._list_tokens(
             'testuserid')
         self.assertEqual(1, len(tokens))
         self.assertIn(token_id, tokens)
@@ -329,31 +344,41 @@ class TokenTests(object):
                        'token_data': {'token': {
                            'audit_ids': [uuid.uuid4().hex]}}}
         # Create 2 Tokens.
-        self.token_provider_api._persistence.create_token(token_id,
-                                                          token_data)
-        self.token_provider_api._persistence.create_token(token2_id,
-                                                          token2_data)
+        PROVIDERS.token_provider_api._persistence.create_token(
+            token_id, token_data
+        )
+        PROVIDERS.token_provider_api._persistence.create_token(
+            token2_id, token2_data
+        )
         # Verify the revocation list is empty.
         self.assertEqual(
-            [], self.token_provider_api._persistence.list_revoked_tokens())
-        self.assertEqual([], self.token_provider_api.list_revoked_tokens())
+            [], PROVIDERS.token_provider_api._persistence.list_revoked_tokens()
+        )
+        self.assertEqual(
+            [], PROVIDERS.token_provider_api.list_revoked_tokens()
+        )
         # Delete a token directly, bypassing the manager.
-        self.token_provider_api._persistence.driver.delete_token(token_id)
+        PROVIDERS.token_provider_api._persistence.driver.delete_token(token_id)
         # Verify the revocation list is still empty.
         self.assertEqual(
-            [], self.token_provider_api._persistence.list_revoked_tokens())
-        self.assertEqual([], self.token_provider_api.list_revoked_tokens())
+            [], PROVIDERS.token_provider_api._persistence.list_revoked_tokens()
+        )
+        self.assertEqual(
+            [], PROVIDERS.token_provider_api.list_revoked_tokens()
+        )
         # Invalidate the revocation list.
-        self.token_provider_api._persistence.invalidate_revocation_list()
+        PROVIDERS.token_provider_api._persistence.invalidate_revocation_list()
         # Verify the deleted token is in the revocation list.
-        revoked_ids = [x['id']
-                       for x in self.token_provider_api.list_revoked_tokens()]
+        revoked_ids = [
+            x['id'] for x in PROVIDERS.token_provider_api.list_revoked_tokens()
+        ]
         self._assert_revoked_token_list_matches_token_persistence(revoked_ids)
         self.assertIn(token_id, revoked_ids)
         # Delete the second token, through the manager
-        self.token_provider_api._persistence.delete_token(token2_id)
-        revoked_ids = [x['id']
-                       for x in self.token_provider_api.list_revoked_tokens()]
+        PROVIDERS.token_provider_api._persistence.delete_token(token2_id)
+        revoked_ids = [
+            x['id'] for x in PROVIDERS.token_provider_api.list_revoked_tokens()
+        ]
         self._assert_revoked_token_list_matches_token_persistence(revoked_ids)
         # Verify both tokens are in the revocation list.
         self.assertIn(token_id, revoked_ids)
@@ -364,10 +389,10 @@ class TokenTests(object):
         token = {'user': {'id': uuid.uuid4().hex},
                  'token_data': {'token': {'audit_ids': [uuid.uuid4().hex]}}}
 
-        self.token_provider_api._persistence.create_token(token_id, token)
-        self.token_provider_api._persistence.delete_token(token_id)
+        PROVIDERS.token_provider_api._persistence.create_token(token_id, token)
+        PROVIDERS.token_provider_api._persistence.delete_token(token_id)
 
-        revoked_tokens = self.token_provider_api.list_revoked_tokens()
+        revoked_tokens = PROVIDERS.token_provider_api.list_revoked_tokens()
         revoked_ids = [x['id'] for x in revoked_tokens]
         self._assert_revoked_token_list_matches_token_persistence(revoked_ids)
         self.assertIn(token_id, revoked_ids)
@@ -377,12 +402,12 @@ class TokenTests(object):
     def test_create_unicode_token_id(self):
         token_id = six.text_type(self._create_token_id())
         self.create_token_sample_data(token_id=token_id)
-        self.token_provider_api._persistence.get_token(token_id)
+        PROVIDERS.token_provider_api._persistence.get_token(token_id)
 
     def test_create_unicode_user_id(self):
         user_id = six.text_type(uuid.uuid4().hex)
         token_id, data = self.create_token_sample_data(user_id=user_id)
-        self.token_provider_api._persistence.get_token(token_id)
+        PROVIDERS.token_provider_api._persistence.get_token(token_id)
 
 
 class TokenCacheInvalidation(object):
@@ -390,7 +415,7 @@ class TokenCacheInvalidation(object):
         time = datetime.datetime.utcnow()
         with freezegun.freeze_time(time) as frozen_datetime:
             # Create an equivalent of a scoped token
-            token_id, data = self.token_provider_api.issue_token(
+            token_id, data = PROVIDERS.token_provider_api.issue_token(
                 self.user_foo['id'],
                 ['password'],
                 project_id=self.tenant_bar['id']
@@ -398,7 +423,7 @@ class TokenCacheInvalidation(object):
             self.scoped_token_id = token_id
 
             # ..and an un-scoped one
-            token_id, data = self.token_provider_api.issue_token(
+            token_id, data = PROVIDERS.token_provider_api.issue_token(
                 self.user_foo['id'],
                 ['password']
             )
@@ -407,28 +432,28 @@ class TokenCacheInvalidation(object):
 
             # Validate them, in the various ways possible - this will load the
             # responses into the token cache.
-            self.token_provider_api.validate_token(self.scoped_token_id)
-            self.token_provider_api.validate_token(self.unscoped_token_id)
+            PROVIDERS.token_provider_api.validate_token(self.scoped_token_id)
+            PROVIDERS.token_provider_api.validate_token(self.unscoped_token_id)
 
     def test_delete_unscoped_token(self):
         time = datetime.datetime.utcnow()
         with freezegun.freeze_time(time) as frozen_datetime:
-            self.token_provider_api._persistence.delete_token(
+            PROVIDERS.token_provider_api._persistence.delete_token(
                 self.unscoped_token_id)
             frozen_datetime.tick(delta=datetime.timedelta(seconds=1))
 
             # Ensure the unscoped token is invalid
             self.assertRaises(
                 exception.TokenNotFound,
-                self.token_provider_api.validate_token,
+                PROVIDERS.token_provider_api.validate_token,
                 self.unscoped_token_id)
             # Ensure the scoped token is still valid
-            self.token_provider_api.validate_token(self.scoped_token_id)
+            PROVIDERS.token_provider_api.validate_token(self.scoped_token_id)
 
     def test_delete_scoped_token_by_id(self):
         time = datetime.datetime.utcnow()
         with freezegun.freeze_time(time) as frozen_datetime:
-            self.token_provider_api._persistence.delete_token(
+            PROVIDERS.token_provider_api._persistence.delete_token(
                 self.scoped_token_id
             )
             frozen_datetime.tick(delta=datetime.timedelta(seconds=1))
@@ -436,15 +461,15 @@ class TokenCacheInvalidation(object):
             # Ensure the project token is invalid
             self.assertRaises(
                 exception.TokenNotFound,
-                self.token_provider_api.validate_token,
+                PROVIDERS.token_provider_api.validate_token,
                 self.scoped_token_id)
             # Ensure the unscoped token is still valid
-            self.token_provider_api.validate_token(self.unscoped_token_id)
+            PROVIDERS.token_provider_api.validate_token(self.unscoped_token_id)
 
     def test_delete_scoped_token_by_user(self):
         time = datetime.datetime.utcnow()
         with freezegun.freeze_time(time) as frozen_datetime:
-            self.token_provider_api._persistence.delete_tokens(
+            PROVIDERS.token_provider_api._persistence.delete_tokens(
                 self.user_foo['id']
             )
             frozen_datetime.tick(delta=datetime.timedelta(seconds=1))
@@ -453,17 +478,17 @@ class TokenCacheInvalidation(object):
             # now be invalid.
             self.assertRaises(
                 exception.TokenNotFound,
-                self.token_provider_api.validate_token,
+                PROVIDERS.token_provider_api.validate_token,
                 self.scoped_token_id)
             self.assertRaises(
                 exception.TokenNotFound,
-                self.token_provider_api.validate_token,
+                PROVIDERS.token_provider_api.validate_token,
                 self.unscoped_token_id)
 
     def test_delete_scoped_token_by_user_and_tenant(self):
         time = datetime.datetime.utcnow()
         with freezegun.freeze_time(time) as frozen_datetime:
-            self.token_provider_api._persistence.delete_tokens(
+            PROVIDERS.token_provider_api._persistence.delete_tokens(
                 self.user_foo['id'],
                 tenant_id=self.tenant_bar['id'])
             frozen_datetime.tick(delta=datetime.timedelta(seconds=1))
@@ -471,7 +496,7 @@ class TokenCacheInvalidation(object):
             # Ensure the scoped token is invalid
             self.assertRaises(
                 exception.TokenNotFound,
-                self.token_provider_api.validate_token,
+                PROVIDERS.token_provider_api.validate_token,
                 self.scoped_token_id)
             # Ensure the unscoped token is still valid
-            self.token_provider_api.validate_token(self.unscoped_token_id)
+            PROVIDERS.token_provider_api.validate_token(self.unscoped_token_id)
