@@ -22,7 +22,6 @@ from oslo_utils import timeutils
 from keystone.common import sql
 import keystone.conf
 from keystone import exception
-from keystone.i18n import _LI
 from keystone import token
 from keystone.token.providers import common
 
@@ -31,7 +30,7 @@ CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 
-class TokenModel(sql.ModelBase, sql.DictBase):
+class TokenModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'token'
     attributes = ['id', 'expires', 'user_id', 'trust_id']
     id = sql.Column(sql.String(64), primary_key=True)
@@ -304,4 +303,4 @@ class Token(token.persistence.TokenDriverBase):
             # When the "with: " block ends, the final "session.commit()"
             # is emitted by enginefacade
             session.flush()
-            LOG.info(_LI('Total expired tokens removed: %d'), total_removed)
+            LOG.info('Total expired tokens removed: %d', total_removed)
