@@ -22,18 +22,18 @@ from oslo_utils import timeutils
 import six
 
 from keystone.common import cache
-from keystone.common import dependency
 from keystone.common import manager
+from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 from keystone.models import token_model
 from keystone import notifications
-from keystone.token import persistence
 
 
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
+PROVIDERS = provider_api.ProviderAPIs
 
 TOKENS_REGION = cache.create_region(name='tokens')
 MEMOIZE_TOKENS = cache.get_memoization_decorator(
@@ -45,13 +45,10 @@ MEMOIZE_TOKENS = cache.get_memoization_decorator(
 UnsupportedTokenVersionException = exception.UnsupportedTokenVersionException
 
 # supported token versions
-V2 = token_model.V2
 V3 = token_model.V3
 VERSIONS = token_model.VERSIONS
 
 
-@dependency.provider('token_provider_api')
-@dependency.requires('assignment_api', 'revoke_api')
 class Manager(manager.Manager):
     """Default pivot point for the token provider backend.
 
@@ -61,8 +58,8 @@ class Manager(manager.Manager):
     """
 
     driver_namespace = 'keystone.token.provider'
+    _provides_api = 'token_provider_api'
 
-    V2 = V2
     V3 = V3
     VERSIONS = VERSIONS
     INVALIDATE_PROJECT_TOKEN_PERSISTENCE = 'invalidate_project_tokens'
@@ -94,6 +91,8 @@ class Manager(manager.Manager):
                     self._delete_user_project_tokens_callback],
                 [notifications.INVALIDATE_USER_OAUTH_CONSUMER_TOKENS,
                     self._delete_user_oauth_consumer_tokens_callback],
+                [notifications.INVALIDATE_TOKEN_CACHE_DELETED_IDP,
+                    self._invalidate_token_cache_from_deleted_idp_callback],
             ]
         }
 
@@ -113,7 +112,7 @@ class Manager(manager.Manager):
         # the provider manager requires the token persistence manager, which
         # requires the token provider manager).
         if self._persistence_manager is None:
-            self._persistence_manager = persistence.PersistenceManager()
+            self._persistence_manager = self._token_persistence_manager
         return self._persistence_manager
 
     def _create_token(self, token_id, token_data):
@@ -131,30 +130,16 @@ class Manager(manager.Manager):
             except exception.TokenNotFound:
                 six.reraise(*exc_info)
 
-    def check_revocation_v2(self, token):
-        try:
-            token_data = token['access']
-        except KeyError:
-            raise exception.TokenNotFound(_('Failed to validate token'))
-
-        token_values = self.revoke_api.model.build_token_values_v2(
-            token_data, CONF.identity.default_domain_id)
-        self.revoke_api.check_token(token_values)
-
     def check_revocation_v3(self, token):
         try:
             token_data = token['token']
         except KeyError:
             raise exception.TokenNotFound(_('Failed to validate token'))
         token_values = self.revoke_api.model.build_token_values(token_data)
-        self.revoke_api.check_token(token_values)
+        PROVIDERS.revoke_api.check_token(token_values)
 
     def check_revocation(self, token):
-        version = self.get_token_version(token)
-        if version == V2:
-            return self.check_revocation_v2(token)
-        else:
-            return self.check_revocation_v3(token)
+        return self.check_revocation_v3(token)
 
     def validate_token(self, token_id, window_seconds=0):
         if not token_id:
@@ -213,12 +198,16 @@ class Manager(manager.Manager):
             raise exception.TokenNotFound(_('Failed to validate token'))
 
     def issue_token(self, user_id, method_names, expires_at=None,
-                    project_id=None, is_domain=False, domain_id=None,
-                    auth_context=None, trust=None, include_catalog=True,
+                    system=None, project_id=None, is_domain=False,
+                    domain_id=None, auth_context=None, trust=None,
+                    app_cred_id=None, include_catalog=True,
                     parent_audit_id=None):
         token_id, token_data = self.driver.issue_token(
-            user_id, method_names, expires_at, project_id, domain_id,
-            auth_context, trust, include_catalog, parent_audit_id)
+            user_id, method_names, expires_at=expires_at,
+            system=system, project_id=project_id,
+            domain_id=domain_id, auth_context=auth_context, trust=trust,
+            app_cred_id=app_cred_id, include_catalog=include_catalog,
+            parent_audit_id=parent_audit_id)
 
         if self._needs_persistence:
             data = dict(key=token_id,
@@ -261,11 +250,12 @@ class Manager(manager.Manager):
         domain_id = token_ref.domain_id if token_ref.domain_scoped else None
 
         if revoke_chain:
-            self.revoke_api.revoke_by_audit_chain_id(token_ref.audit_chain_id,
-                                                     project_id=project_id,
-                                                     domain_id=domain_id)
+            PROVIDERS.revoke_api.revoke_by_audit_chain_id(
+                token_ref.audit_chain_id, project_id=project_id,
+                domain_id=domain_id
+            )
         else:
-            self.revoke_api.revoke_by_audit_id(token_ref.audit_id)
+            PROVIDERS.revoke_api.revoke_by_audit_id(token_ref.audit_id)
 
         if CONF.token.revoke_by_id and self._needs_persistence:
             self._persistence.delete_token(token_id=token_id)
@@ -283,7 +273,7 @@ class Manager(manager.Manager):
                                       payload):
         if CONF.token.revoke_by_id:
             trust_id = payload['resource_info']
-            trust = self.trust_api.get_trust(trust_id, deleted=True)
+            trust = PROVIDERS.trust_api.get_trust(trust_id, deleted=True)
             self._persistence.delete_tokens(user_id=trust['trustor_user_id'],
                                             trust_id=trust_id)
         if CONF.token.cache_on_issue:
@@ -324,7 +314,7 @@ class Manager(manager.Manager):
         if CONF.token.revoke_by_id:
             project_id = payload['resource_info']
             self._persistence.delete_tokens_for_users(
-                self.assignment_api.list_user_ids_for_project(project_id),
+                PROVIDERS.assignment_api.list_user_ids_for_project(project_id),
                 project_id=project_id)
         if CONF.token.cache_on_issue:
             # NOTE(amakarov): preserving behavior
@@ -341,3 +331,16 @@ class Manager(manager.Manager):
         if CONF.token.cache_on_issue:
             # NOTE(amakarov): preserving behavior
             TOKENS_REGION.invalidate()
+
+    def _invalidate_token_cache_from_deleted_idp_callback(self,
+                                                          service,
+                                                          resource_type,
+                                                          operation, payload):
+        """Callback to invalidate the token cache after deleting an idp.
+
+        While this callback doesn't use the information about the deleted
+        identity provider to invalidate the cache, the method name and payload
+        are emitted when logging at the DEBUG level.
+
+        """
+        TOKENS_REGION.invalidate()
