@@ -19,22 +19,18 @@
 # This script is based on the original DevStack keystone_data.sh script.
 #
 # It demonstrates how to bootstrap Keystone with an administrative user
-# using the OS_TOKEN and OS_URL environment variables and the administrative
-# API.  It will get the admin_token (OS_TOKEN) and admin_port from
-# keystone.conf if available.
+# using the `keystone-manage bootstrap` command.  It will get the admin_port
+# from keystone.conf if available.
 #
 # Disable creation of endpoints by setting DISABLE_ENDPOINTS environment variable.
 # Use this with the Catalog Templated backend.
 #
-# A EC2-compatible credential is created for the admin user and
-# placed in etc/ec2rc.
-#
-# Tenant               User      Roles
+# Project              User      Roles
 # -------------------------------------------------------
 # demo                 admin     admin
 # service              glance    service
 # service              nova      service
-# service              ec2       service
+# service              cinder    service
 # service              swift     service
 # service              neutron   service
 
@@ -49,10 +45,10 @@ type openstack >/dev/null 2>&1 || {
     exit 1
     }
 
-ADMIN_PASSWORD=${ADMIN_PASSWORD:-secrete}
+ADMIN_PASSWORD=${ADMIN_PASSWORD:-secret}
 NOVA_PASSWORD=${NOVA_PASSWORD:-${SERVICE_PASSWORD:-nova}}
 GLANCE_PASSWORD=${GLANCE_PASSWORD:-${SERVICE_PASSWORD:-glance}}
-EC2_PASSWORD=${EC2_PASSWORD:-${SERVICE_PASSWORD:-ec2}}
+CINDER_PASSWORD=${CINDER_PASSWORD:-${SERVICE_PASSWORD:-cinder}}
 SWIFT_PASSWORD=${SWIFT_PASSWORD:-${SERVICE_PASSWORD:-swiftpass}}
 NEUTRON_PASSWORD=${NEUTRON_PASSWORD:-${SERVICE_PASSWORD:-neutron}}
 
@@ -62,24 +58,17 @@ CONTROLLER_INTERNAL_ADDRESS=${CONTROLLER_INTERNAL_ADDRESS:-localhost}
 
 TOOLS_DIR=$(cd $(dirname "$0") && pwd)
 KEYSTONE_CONF=${KEYSTONE_CONF:-/etc/keystone/keystone.conf}
-if [[ -r "$KEYSTONE_CONF" ]]; then
-    EC2RC="$(dirname "$KEYSTONE_CONF")/ec2rc"
-elif [[ -r "$TOOLS_DIR/../etc/keystone.conf" ]]; then
-    # assume git checkout
-    KEYSTONE_CONF="$TOOLS_DIR/../etc/keystone.conf"
-    EC2RC="$TOOLS_DIR/../etc/ec2rc"
-else
-    KEYSTONE_CONF=""
-    EC2RC="ec2rc"
+if [[ ! -r "$KEYSTONE_CONF" ]]; then
+    if [[ -r "$TOOLS_DIR/../etc/keystone.conf" ]]; then
+        # assume git checkout
+        KEYSTONE_CONF="$TOOLS_DIR/../etc/keystone.conf"
+    else
+        KEYSTONE_CONF=""
+    fi
 fi
 
 # Extract some info from Keystone's configuration file
 if [[ -r "$KEYSTONE_CONF" ]]; then
-    CONFIG_SERVICE_TOKEN=$(sed 's/[[:space:]]//g' $KEYSTONE_CONF | grep ^admin_token= | cut -d'=' -f2)
-    if [[ -z "${CONFIG_SERVICE_TOKEN}" ]]; then
-        # default config options are commented out, so lets try those
-        CONFIG_SERVICE_TOKEN=$(sed 's/[[:space:]]//g' $KEYSTONE_CONF | grep ^\#admin_token= | cut -d'=' -f2)
-    fi
     CONFIG_ADMIN_PORT=$(sed 's/[[:space:]]//g' $KEYSTONE_CONF | grep ^admin_port= | cut -d'=' -f2)
     if [[ -z "${CONFIG_ADMIN_PORT}" ]]; then
         # default config options are commented out, so lets try those
@@ -87,26 +76,20 @@ if [[ -r "$KEYSTONE_CONF" ]]; then
     fi
 fi
 
-export OS_TOKEN=${OS_TOKEN:-$CONFIG_SERVICE_TOKEN}
-if [[ -z "$OS_TOKEN" ]]; then
-    echo "No service token found."
-    echo "Set OS_TOKEN manually from keystone.conf admin_token."
-    exit 1
-fi
+export OS_USERNAME=admin
+export OS_PASSWORD=$ADMIN_PASSWORD
+export OS_PROJECT_NAME=admin
+export OS_USER_DOMAIN_ID=default
+export OS_PROJECT_DOMAIN_ID=default
+export OS_IDENTITY_API_VERSION=3
+export OS_AUTH_URL=http://$CONTROLLER_PUBLIC_ADDRESS:${CONFIG_ADMIN_PORT:-35357}/v3
 
-export OS_URL=${OS_URL:-http://$CONTROLLER_PUBLIC_ADDRESS:${CONFIG_ADMIN_PORT:-35357}/v2.0}
-
-function get_id () {
-    echo `"$@" | grep ' id ' | awk '{print $4}'`
-}
-
-#
-# Roles
-#
-
-openstack role create admin
-
-openstack role create service
+export OS_BOOTSTRAP_PASSWORD=$ADMIN_PASSWORD
+export OS_BOOTSTRAP_REGION_ID=RegionOne
+export OS_BOOTSTRAP_ADMIN_URL="http://$CONTROLLER_PUBLIC_ADDRESS:\$(public_port)s/v3"
+export OS_BOOTSTRAP_PUBLIC_URL="http://$CONTROLLER_ADMIN_ADDRESS:\$(admin_port)s/v3"
+export OS_BOOTSTRAP_INTERNAL_URL="http://$CONTROLLER_INTERNAL_ADDRESS:\$(public_port)s/v3"
+keystone-manage bootstrap
 
 #
 # Default tenant
@@ -114,16 +97,11 @@ openstack role create service
 openstack project create demo \
                          --description "Default Tenant"
 
-openstack user create admin --project demo \
-                      --password "${ADMIN_PASSWORD}"
-
-openstack role add --user admin \
-                   --project demo\
-                   admin
-
 #
 # Service tenant
 #
+openstack role create service
+
 openstack project create service \
                   --description "Service Tenant"
 
@@ -141,10 +119,10 @@ openstack role add --user nova \
                    --project service \
                    service
 
-openstack user create ec2 --project service \
-                      --password "${EC2_PASSWORD}"
+openstack user create cinder --project service \
+                      --password "${CINDER_PASSWORD}"
 
-openstack role add --user ec2 \
+openstack role add --user cinder \
                    --project service \
                    service
 
@@ -162,46 +140,58 @@ openstack role add --user neutron \
                    --project service \
                    service
 
-#
-# Keystone service
-#
-openstack service create --name keystone \
-                         --description "Keystone Identity Service" \
-                         identity
-if [[ -z "$DISABLE_ENDPOINTS" ]]; then
-    openstack endpoint create --region RegionOne \
-        --publicurl "http://$CONTROLLER_PUBLIC_ADDRESS:\$(public_port)s/v2.0" \
-        --adminurl "http://$CONTROLLER_ADMIN_ADDRESS:\$(admin_port)s/v2.0" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:\$(public_port)s/v2.0" \
-        keystone
-fi
-
 #
 # Nova service
 #
+openstack service create --name=nova_legacy \
+                         --description="Nova Compute Service (Legacy 2.0)" \
+                         compute_legacy
+if [[ -z "$DISABLE_ENDPOINTS" ]]; then
+    openstack endpoint create --region RegionOne \
+        compute public "http://$CONTROLLER_PUBLIC_ADDRESS:8774/v2/\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        compute admin "http://$CONTROLLER_ADMIN_ADDRESS:8774/v2/\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        compute internal "http://$CONTROLLER_INTERNAL_ADDRESS:8774/v2/\$(project_id)s"
+fi
+
 openstack service create --name=nova \
                          --description="Nova Compute Service" \
                          compute
 if [[ -z "$DISABLE_ENDPOINTS" ]]; then
     openstack endpoint create --region RegionOne \
-        --publicurl "http://$CONTROLLER_PUBLIC_ADDRESS:8774/v2/\$(tenant_id)s" \
-        --adminurl "http://$CONTROLLER_ADMIN_ADDRESS:8774/v2/\$(tenant_id)s" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:8774/v2/\$(tenant_id)s" \
-        nova
+        compute public "http://$CONTROLLER_PUBLIC_ADDRESS:8774/v2.1"
+    openstack endpoint create --region RegionOne \
+        compute admin "http://$CONTROLLER_ADMIN_ADDRESS:8774/v2.1"
+    openstack endpoint create --region RegionOne \
+        compute internal "http://$CONTROLLER_INTERNAL_ADDRESS:8774/v2.1"
 fi
 
 #
 # Volume service
 #
-openstack service create --name=volume \
-                         --description="Cinder Volume Service" \
-                         volume
+openstack service create --name=cinderv2 \
+                         --description="Cinder Volume Service V2" \
+                         volumev2
 if [[ -z "$DISABLE_ENDPOINTS" ]]; then
     openstack endpoint create --region RegionOne \
-        --publicurl "http://$CONTROLLER_PUBLIC_ADDRESS:8776/v1/\$(tenant_id)s" \
-        --adminurl "http://$CONTROLLER_ADMIN_ADDRESS:8776/v1/\$(tenant_id)s" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:8776/v1/\$(tenant_id)s" \
-        volume
+        volume public "http://$CONTROLLER_PUBLIC_ADDRESS:8776/v2/\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        volume admin "http://$CONTROLLER_ADMIN_ADDRESS:8776/v2/\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        volume internal "http://$CONTROLLER_INTERNAL_ADDRESS:8776/v2/\$(project_id)s"
+fi
+
+openstack service create --name=cinderv3 \
+                         --description="Cinder Volume Service V3" \
+                         volumev3
+if [[ -z "$DISABLE_ENDPOINTS" ]]; then
+    openstack endpoint create --region RegionOne \
+        volume public "http://$CONTROLLER_PUBLIC_ADDRESS:8776/v3/\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        volume admin "http://$CONTROLLER_ADMIN_ADDRESS:8776/v3/\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        volume internal "http://$CONTROLLER_INTERNAL_ADDRESS:8776/v3/\$(project_id)s"
 fi
 
 #
@@ -212,24 +202,11 @@ openstack service create --name=glance \
                          image
 if [[ -z "$DISABLE_ENDPOINTS" ]]; then
     openstack endpoint create --region RegionOne  \
-        --publicurl "http://$CONTROLLER_PUBLIC_ADDRESS:9292" \
-        --adminurl "http://$CONTROLLER_ADMIN_ADDRESS:9292" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:9292" \
-        glance
-fi
-
-#
-# EC2 service
-#
-openstack service create --name=ec2 \
-                         --description="EC2 Compatibility Layer" \
-                         ec2
-if [[ -z "$DISABLE_ENDPOINTS" ]]; then
-    openstack endpoint create --region RegionOne \
-        --publicurl "http://$CONTROLLER_PUBLIC_ADDRESS:8773/services/Cloud" \
-        --adminurl "http://$CONTROLLER_ADMIN_ADDRESS:8773/services/Admin" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:8773/services/Cloud" \
-        ec2
+        image public "http://$CONTROLLER_PUBLIC_ADDRESS:9292"
+    openstack endpoint create --region RegionOne  \
+        image admin "http://$CONTROLLER_ADMIN_ADDRESS:9292"
+    openstack endpoint create --region RegionOne  \
+        image internal "http://$CONTROLLER_INTERNAL_ADDRESS:9292"
 fi
 
 #
@@ -240,10 +217,11 @@ openstack service create --name=swift \
                          object-store
 if [[ -z "$DISABLE_ENDPOINTS" ]]; then
     openstack endpoint create --region RegionOne \
-        --publicurl   "http://$CONTROLLER_PUBLIC_ADDRESS:8080/v1/AUTH_\$(tenant_id)s" \
-        --adminurl    "http://$CONTROLLER_ADMIN_ADDRESS:8080/v1" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:8080/v1/AUTH_\$(tenant_id)s" \
-        swift
+        object-store public "http://$CONTROLLER_PUBLIC_ADDRESS:8080/v1/AUTH_\$(project_id)s"
+    openstack endpoint create --region RegionOne \
+        object-store admin "http://$CONTROLLER_ADMIN_ADDRESS:8080/v1"
+    openstack endpoint create --region RegionOne \
+        object-store internal "http://$CONTROLLER_INTERNAL_ADDRESS:8080/v1/AUTH_\$(project_id)s"
 fi
 
 #
@@ -254,20 +232,9 @@ openstack service create --name=neutron \
                          network
 if [[ -z "$DISABLE_ENDPOINTS" ]]; then
     openstack endpoint create --region RegionOne \
-        --publicurl   "http://$CONTROLLER_PUBLIC_ADDRESS:9696" \
-        --adminurl    "http://$CONTROLLER_ADMIN_ADDRESS:9696" \
-        --internalurl "http://$CONTROLLER_INTERNAL_ADDRESS:9696" \
-        neutron
+        network public "http://$CONTROLLER_PUBLIC_ADDRESS:9696"
+    openstack endpoint create --region RegionOne \
+        network admin "http://$CONTROLLER_ADMIN_ADDRESS:9696"
+    openstack endpoint create --region RegionOne \
+        network internal "http://$CONTROLLER_INTERNAL_ADDRESS:9696"
 fi
-
-# create ec2 creds and parse the secret and access key returned
-ADMIN_USER=$(get_id openstack user show admin)
-RESULT=$(openstack ec2 credentials create --project service --user $ADMIN_USER)
-ADMIN_ACCESS=`echo "$RESULT" | grep access | awk '{print $4}'`
-ADMIN_SECRET=`echo "$RESULT" | grep secret | awk '{print $4}'`
-
-# write the secret and access to ec2rc
-cat > $EC2RC <<EOF
-ADMIN_ACCESS=$ADMIN_ACCESS
-ADMIN_SECRET=$ADMIN_SECRET
-EOF
